import configparser
import os
import sys

from PtpUploader import Ptp
from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.Settings import Settings


def LoadNotifierSettings():
    configParser = configparser.ConfigParser()
    configParser.optionxform = str  # Make option names case sensitive.

    # Load Notifier.ini from the same directory where PtpUploader is.
    settingsDirectory, _ = os.path.split(
        __file__
    )  # __file__ contains the full path of the current running module
    settingsPath = os.path.join(settingsDirectory, "Notifier.ini")
    configParser.read(settingsPath)

    return configParser.get("Settings", "UserId")


def Notify(releaseName, uploadedTorrentUrl):
    userId = LoadNotifierSettings()
    userId = userId.strip()
    if not userId.isdigit():
        return

    Ptp.Login()
    subject = "[PtpUploader] %s" % releaseName
    message = (
        "This is an automatic notification about a new [url=%s]upload[/url]."
        % uploadedTorrentUrl
    )
    Ptp.SendPrivateMessage(userId, subject, message)


if __name__ == "__main__":
    Settings.LoadSettings()
    MyGlobals.InitializeGlobals(Settings.WorkingPath)

    if len(sys.argv) == 3:
        Notify(sys.argv[1], sys.argv[2])



import logging
import os
import time
import urllib

from datetime import datetime
from pathlib import Path
from typing import Any, Dict

import dynaconf

from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse, HttpResponseRedirect, JsonResponse
from django.shortcuts import get_object_or_404, render
from django.templatetags.static import static
from django.urls import reverse
from django.utils import timezone
from django.utils.html import escape as html_escape
from django.views.decorators.csrf import csrf_exempt

from PtpUploader import Ptp, nfo_parser
from PtpUploader.Helper import SizeToText, TimeDifferenceToText
from PtpUploader.Job.JobRunningState import JobRunningState
from PtpUploader.Job.JobStartMode import JobStartMode
from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.PtpUploaderMessage import (
    PtpUploaderMessageDeleteJob,
    PtpUploaderMessageStartJob,
    PtpUploaderMessageStopJob,
)
from PtpUploader.ReleaseInfo import ReleaseInfo
from PtpUploader.Settings import Settings, config

from . import forms


logger = logging.getLogger(__name__)


def GetStateIcon(state: int) -> str:
    if state == ReleaseInfo.JobState.InDownload:
        return '<div class="download-spinner"></div>'
    if state == ReleaseInfo.JobState.InProgress:
        return '<div class="inprogress-spinner"></div>'
    if state in [
        ReleaseInfo.JobState.Ignored,
        ReleaseInfo.JobState.Ignored_AlreadyExists,
        ReleaseInfo.JobState.Ignored_Forbidden,
        ReleaseInfo.JobState.Ignored_MissingInfo,
        ReleaseInfo.JobState.Ignored_NotSupported,
    ]:
        return '<span class="icon"><i class="fas fa-exclamation-triangle has-text-warning"></i></span>'
    i = {
        ReleaseInfo.JobState.Finished: "fa-check has-text-success",
        ReleaseInfo.JobState.Failed: "fa-exclamation-circle has-text-danger",
        ReleaseInfo.JobState.WaitingForStart: "fa-hourglass has-text-info",
        ReleaseInfo.JobState.Paused: "fa-pause has-text-info",
        ReleaseInfo.JobState.DownloadedAlreadyExists: "fa-frown has-text-warning",
        ReleaseInfo.JobState.InProgress: "fa-spinner fa-pulse has-text-info",
        ReleaseInfo.JobState.InDownload: "spinner",
        ReleaseInfo.JobState.Scheduled: "fa-clock",
    }
    return f'<span class="icon"><i class="fas {i[state]}"></i></span>'


@login_required
def jobs(request):
    return render(request, "jobs.html", {"state": ReleaseInfo.JobState.choices})


@login_required
def log(_request, r_id: int):
    get_object_or_404(ReleaseInfo, Id=r_id)

    log_msg = ""
    logFilePath = os.path.join(Settings.GetJobLogPath(), str(r_id))

    if os.path.isfile(logFilePath):
        with open(logFilePath) as fh:
            log_msg = fh.read()
    else:
        log_msg = "Log file '%s' doesn't exists!" % logFilePath

    log_msg = log_msg.replace("\n", r"<br>")

    return HttpResponse(log_msg)


def GetPtpOrImdbId(releaseInfo, text):
    imdbId = nfo_parser.get_imdb_id(text)
    if len(imdbId) > 0:
        releaseInfo.ImdbId = imdbId
    elif text in ["0", "-"]:
        releaseInfo.ImdbId = "0"
    else:
        # Using urlparse because of torrent permalinks:
        # https://passthepopcorn.me/torrents.php?id=9730&torrentid=72322
        url = urllib.parse.urlparse(text)
        if (
            url.netloc == "passthepopcorn.me"
            or url.netloc == "www.passthepopcorn.me"
            or url.netloc == "tls.passthepopcorn.me"
        ):
            params = urllib.parse.parse_qs(url.query)
            ptpIdList = params.get("id")
            if ptpIdList is not None:
                releaseInfo.PtpId = ptpIdList[0]
                releaseInfo.ImdbId = Ptp.GetMoviePageOnPtp(
                    logger, releaseInfo.PtpId
                ).ImdbId


@login_required
def jobs_get_latest(request):
    releaseInfo = ReleaseInfo()
    GetPtpOrImdbId(releaseInfo, request.GET["PtpOrImdbLink"])

    torrentId = 0

    if releaseInfo.ImdbId != "0":
        Ptp.Login()

        movieOnPtpResult = None
        if releaseInfo.PtpId:
            movieOnPtpResult = Ptp.GetMoviePageOnPtp(logger, releaseInfo.PtpId)
        else:
            movieOnPtpResult = Ptp.GetMoviePageOnPtpByImdbId(logger, releaseInfo.ImdbId)

        if movieOnPtpResult:
            torrent = movieOnPtpResult.GetLatestTorrent()
            if torrent:
                torrentId = torrent["Id"]

                difference = datetime.now() - torrent["UploadTime"]

    return JsonResponse(
        {
            "Result": "OK",
            "TorrentId": torrentId,
            "UploadedAgo": TimeDifferenceToText(difference).lower(),
        }
    )


@login_required
def settings(request):
    if request.method == "POST":
        form = forms.SettingsForm(request.POST)
        if form.is_valid():
            config.image_host.use = form.cleaned_data["image_host_use"]
            config.ptp.username = form.cleaned_data["ptp_username"]
            config.ptp.password = form.cleaned_data["ptp_password"]
            config.ptp.announce_url = form.cleaned_data["ptp_announce_url"]
            dynaconf.loaders.yaml_loader.write(
                Path("~/.config/ptpuploader/config.yml").expanduser(),
                {
                    "ptp": {
                        "username": form.cleaned_data["ptp_username"],
                        "password": form.cleaned_data["ptp_password"],
                        "announce_url": form.cleaned_data["ptp_announce_url"],
                    },
                    "client": {
                        "use": form.cleaned_data["client_use"],
                        form.cleaned_data["client_use"]: {
                            "address": form.cleaned_data["client_address"]
                        },
                    },
                    "image_host": {"use": form.cleaned_data["image_host_use"]},
                },
            )
    else:
        form = forms.SettingsForm()
    return render(request, "settings.html", {"form": form})


@login_required
def bulk_upload(request):
    if request.method == "POST":
        # create a form instance and populate it with data from the request:
        form = forms.BulkReleaseForm(request.POST)
        # check whether it's valid:
        if form.is_valid():
            for link in form.cleaned_data["Links"].split("\n"):
                release = ReleaseInfo(AnnouncementId=link)
                release.JobStartMode = JobStartMode.Manual
                if "post_stop_before" in request.POST:
                    release.StopBeforeUploading = True
                else:
                    release.StopBeforeUploading = False
                release.save()
            for path in form.cleaned_data["Paths"].split("\n"):
                if not path.strip():
                    continue
                path = Path(path)
                release = ReleaseInfo()
                release.AnnouncementSourceName = "file"
                release.ReleaseDownloadPath = path
                release.ReleaseName = path.name
                release.JobStartMode = JobStartMode.Manual
                if "post_stop_before" in request.POST:
                    release.StopBeforeUploading = True
                else:
                    release.StopBeforeUploading = False
                release.save()
            # process the data in form.cleaned_data as required
            # ...
            # redirect to a new URL:
            return HttpResponseRedirect("/jobs")

    # if a GET (or any other method) we'll create a blank form
    else:
        form = forms.BulkReleaseForm()

    return render(request, "bulk.html", {"form": form})


@login_required
def jobs_json(_):
    settings = {}

    entries = []
    for release in ReleaseInfo.objects.all():
        # Preprocess some values for consistent formatting
        entry = {}
        for field in (
            "Id",
            "PtpId",
            "ImdbId",
            "ErrorMessage",
            "ReleaseName",
            "PtpTorrentId",
        ):
            entry[field] = getattr(release, field)
        entry["Size"] = {
            "sort": int(release.Size),
            "_": SizeToText(release.Size),
        }
        entry["LastModificationTime"] = {
            "sort": int(release.LastModificationTime.strftime("%s")),
            "_": TimeDifferenceToText(
                timezone.now() - release.LastModificationTime,
                2,
            ),
        }

        # Build icons and links
        source = MyGlobals.SourceFactory.GetSource(release.AnnouncementSourceName)
        if source is not None:
            icon = static(f"source_icon/{source.Name}.ico")
            entry["AnnouncementSourceName"] = {
                "sort": release.AnnouncementSourceName,
                "_": f'<img src="{icon}">',
            }
            url = source.GetUrlFromId(release.AnnouncementId)
            if url:
                entry["AnnouncementSourceName"][
                    "_"
                ] = f'<a href="{url}"><img src="{icon}"></a>'
        else:
            entry["AnnouncementSourceName"] = {
                "sort": "",
                "_": "",
            }

        icon = GetStateIcon(release.JobRunningState)
        logUrl = reverse("log", args=[release.Id])
        entry["JobRunningState"] = {
            "sort": release.JobRunningState,
            "_": f'<a href="{logUrl}" title="{release.get_JobRunningState_display()}">{icon}</a>',
        }

        # Build actions
        entry["Actions"] = []
        if release.CanResumed():
            entry["Actions"] += ["start"]
        if release.CanStopped():
            entry["Actions"] += ["stop"]
        if release.CanEdited():
            entry["Actions"] += ["edit"]
        if release.CanDeleted():
            entry["Actions"] += ["delete"]
        entry["Actions"] = (",").join(entry["Actions"])
        entries.append(entry)

        # Format scheduled items with a dynamic sub-message
        if release.JobRunningState == ReleaseInfo.JobState.Scheduled:
            entry["ErrorMessage"] = "Scheduled to  run in " + TimeDifferenceToText(
                release.ScheduleTime - timezone.now(),
                agoText="",
                noDifferenceText="just a moment",
            )
        entry["ErrorMessage"] = html_escape(entry["ErrorMessage"])

    return JsonResponse({"data": entries, "settings": settings})


@login_required
def local_dir(request):
    d: Path
    if "dir" not in request.GET:
        d = Path(config.web.file_selector_root or "~").expanduser()
    else:
        d = Path(request.GET["dir"])
    val = []
    for child in sorted(d.iterdir()):
        c = {"title": child.name, "key": str(child)}
        if child.is_dir():
            c["folder"] = True
            c["lazy"] = True
        elif child.suffix.lower() in [".mkv", ".avi", ".mp4", ".vob", ".ifo", ".bup"]:
            c["icon"] = "film"
        val.append(c)
    return JsonResponse(val, safe=False)  # It's just a list, probably safe


@login_required
def start_job(_, r_id):
    # TODO: This is very far from perfect. There is no guarantee that the job didn't start meanwhile.
    # Probably only the WorkerThread should change the running state.
    releaseInfo = ReleaseInfo.objects.get(Id=r_id)
    if not releaseInfo.CanResumed():
        return HttpResponse("The job is already running!")

    releaseInfo.JobRunningState = JobRunningState.WaitingForStart

    # Make sure that job is no longer handled as an automatically started job.
    # Manual forced jobs will resumed as manual forced.
    if releaseInfo.JobStartMode == JobStartMode.Automatic:
        releaseInfo.JobStartMode = JobStartMode.Manual

    # Resume the job normally.
    releaseInfo.StopBeforeUploading = False

    releaseInfo.save()
    MyGlobals.PtpUploader.add_message(PtpUploaderMessageStartJob(r_id))
    return HttpResponse("OK")


@login_required
def stop_job(_, r_id: int):
    releaseInfo = ReleaseInfo.objects.get(Id=r_id)
    if not releaseInfo.CanStopped():
        return HttpResponse("The job is already stopped!")

    MyGlobals.PtpUploader.add_message(PtpUploaderMessageStopJob(r_id))
    return HttpResponse("OK")


@csrf_exempt
def create(request):
    if (
        request.method != "POST"
        or "password" not in request.POST
        or request.POST["password"] != config.web.api_key
    ):
        raise PermissionDenied
    dest_path = Path(Settings.GetTemporaryPath(), f"{time.time()}.torrent")
    with dest_path.open("wb") as dest:
        for chunk in request.FILES["torrent"].chunks():
            dest.write(chunk)
    release = ReleaseInfo()
    release.SourceTorrentFilePath = dest_path
    release.AnnouncementSourceName = "torrent"
    release.ImdbId = nfo_parser.get_imdb_id(request.POST["imdbUrl"])
    source, source_id = MyGlobals.SourceFactory.GetSourceAndIdByUrl(
        request.POST["SourceUrl"]
    )
    if source is not None:
        release.AnnouncementSourceName = source.Name
        release.AnnouncementId = source_id
    release.JobStartMode = JobStartMode.Manual
    release.StopBeforeUploading = True
    release.save()
    response = JsonResponse({"result": "OK", "jobId": release.Id})
    response["Access-Control-Allow-Origin"] = "*"
    return response


@login_required
def delete_job(_, r_id: int, mode: str):
    releaseInfo = ReleaseInfo.objects.get(Id=r_id)
    if not releaseInfo.CanDeleted():
        return HttpResponse("The job cannot be deleted!")

    MyGlobals.PtpUploader.add_message(PtpUploaderMessageDeleteJob(r_id, mode))
    return HttpResponse("OK")


@login_required
def edit_job(request, r_id: int = -1):
    job: Dict[str, Any] = {
        "id": r_id
    }  # Non-form data for display but too complex for a template
    if r_id > 0:
        release = get_object_or_404(ReleaseInfo, Id=r_id)
    else:
        release = ReleaseInfo()
    if request.method == "POST":
        if "delete" in request.POST:
            if not release.CanDeleted():
                return HttpResponse("The job cannot be deleted!")
            MyGlobals.PtpUploader.add_message(PtpUploaderMessageDeleteJob(r_id, "job"))
            return HttpResponseRedirect("/jobs")
        # create a form instance and populate it with data from the request:
        form = forms.ReleaseForm(request.POST, instance=release)
        # check whether it's valid:
        if form.is_valid():
            form.save()
            release.JobRunningState = JobRunningState.WaitingForStart
            if r_id < 0:
                if request.POST["TorrentLink"]:
                    source, source_id = MyGlobals.SourceFactory.GetSourceAndIdByUrl(
                        request.POST["TorrentLink"]
                    )
                    if source is None:
                        return False

                    release.AnnouncementSourceName = source.Name
                    release.AnnouncementId = source_id
                elif request.POST["LocalFile"]:
                    path = Path(request.POST["LocalFile"])
                    release.AnnouncementSourceName = "file"
                    release.ReleaseDownloadPath = path
                    release.ReleaseName = path.name
                elif request.FILES["RawFile"]:
                    dest_path = Path(
                        Settings.GetTemporaryPath(), f"{time.time()}.torrent"
                    )
                    with dest_path.open("wb") as dest:
                        for chunk in request.FILES["RawFile"].chunks():
                            dest.write(chunk)
                    release.SourceTorrentFilePath = dest_path
                    release.AnnouncementSourceName = "torrent"
            if "post_stop_before" in request.POST:
                release.StopBeforeUploading = True
            else:
                release.StopBeforeUploading = False
            GetPtpOrImdbId(release, release.ImdbId)
            release.save()
            MyGlobals.PtpUploader.add_message(PtpUploaderMessageStartJob(release.Id))
            return HttpResponseRedirect("/jobs")
    # if a GET (or any other method) we'll create a blank form
    else:
        job["Screenshots"] = {}
        if release.Screenshots:
            for f, shots in release.Screenshots.items():
                name = str(Path(f).name)
                job["Screenshots"][name] = ""
                for s in shots:
                    job["Screenshots"][name] += f'<img src="{s}"/>'
        source = MyGlobals.SourceFactory.GetSource(release.AnnouncementSourceName)
        if source is not None:
            filename = "source_icon/%s.ico" % release.AnnouncementSourceName
            job["SourceIcon"] = static(filename)
            job["SourceUrl"] = source.GetUrlFromId(release.AnnouncementId)
        if config.uploader.skip_duplicate_checking:
            release.DuplicateCheckCanIgnore = (
                999999  # Reaching this limit would be one of them good problems
            )
        if release.DuplicateCheckCanIgnore > 0:
            job["SkipDuplicateCheckingButton"] = release.DuplicateCheckCanIgnore
        else:
            job["SkipDuplicateCheckingButton"] = 0
        job["CanEdited"] = release.CanEdited
        job["StopBeforeUploading"] = release.StopBeforeUploading
        job["Status"] = release.get_JobRunningState_display()
        if release.ErrorMessage:
            job["Status"] += f" - {release.ErrorMessage}"

        initial = {
            "Subtitles": release.Subtitles,
            "Tags": release.Tags.split(","),
            "TrumpableNoEnglish": release.TrumpableReasons.NO_ENGLISH_SUBS
            in release.Trumpable,
            "TrumpableHardSubs": release.TrumpableReasons.HARDCODED_SUBS
            in release.Trumpable,
        }
        if release.ImdbId:
            initial["ImdbId"] = f"https://imdb.com/title/tt{release.ImdbId}"
        form = forms.ReleaseForm(instance=release, initial=initial)
    return render(request, "edit_job.html", {"form": form, "settings": {}, "job": job})



"""Handle deletion of jobs when requested by user"""
import logging
import threading

from PtpUploader.Job.WorkerBase import WorkerBase
from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.Settings import Settings


logger = logging.getLogger(__name__)


class Delete(WorkerBase):
    """Create a simple worker with a single phase"""

    def __init__(self, release_id: int, mode: str, stop_requested: threading.Event):
        super().__init__(release_id, stop_requested)
        self.Phases = [self.__delete]
        self.mode = mode

    def __delete(self):
        if not self.ReleaseInfo.CanDeleted():
            logger.error("The job is currently running and can't be deleted!")
            return "Error"

        delete_mode = self.mode.lower()
        delete_source_data = delete_mode in ["job_source", "job_all"]
        delete_upload_data = delete_mode in ["job_upload", "job_all"]

        announcement_source = self.ReleaseInfo.AnnouncementSource
        if announcement_source is None:
            announcement_source = MyGlobals.SourceFactory.GetSource(
                self.ReleaseInfo.AnnouncementSourceName
            )

        if announcement_source is not None:  # Still possibly not there
            announcement_source.Delete(
                self.ReleaseInfo,
                Settings.GetTorrentClient(),
                delete_source_data,
                delete_upload_data,
            )

        self.ReleaseInfo.delete()
        logger.info("Release deleted")

        return "OK"



# Generated by Django 3.2.5 on 2021-10-28 22:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('web', '0003_alter_releaseinfo_jobrunningstate'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='releaseinfo',
            name='Tags',
        ),
        migrations.AddField(
            model_name='releaseinfo',
            name='ForceDirectorylessSingleFileTorrent',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='releaseinfo',
            name='OverrideScreenshots',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='releaseinfo',
            name='PersonalRip',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='releaseinfo',
            name='SceneRelease',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='releaseinfo',
            name='SpecialRelease',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='releaseinfo',
            name='StartImmediately',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='releaseinfo',
            name='StopBeforeUploading',
            field=models.BooleanField(default=False),
        ),
    ]



import logging

import transmissionrpc


class Transmission:
    def __init__(self, address, port):
        logging.getLogger(__name__).info("Initializing transmissionrpc.")
        self.transmission = transmissionrpc.Client(address, port)

    # downloadPath is the final path. Suggested directory name from torrent won't be added to it.
    # Returns with the info hash of the torrent.
    def AddTorrent(self, logger=None, torrentPath=None, downloadPath=None):
        logger.info(
            "Initiating the download of torrent '%s' with Transmission to '%s'."
            % (torrentPath, downloadPath)
        )
        torrent = self.transmission.add_torrent(torrentPath, download_dir=downloadPath)
        return torrent.hashString

    # Transmission doesn't allow hash check skipping...
    def AddTorrentSkipHashCheck(self, logger, torrentPath, downloadPath):
        logger.info(
            "Adding torrent '%s' without hash checking (not really) to Transmission to '%s'."
            % (torrentPath, downloadPath)
        )
        hashString = self.AddTorrent(logger, torrentPath, downloadPath)
        return hashString

    def IsTorrentFinished(self, logger, infoHash):
        try:
            # TODO: not the most sophisticated way.
            # Even a watch dir with Pyinotify would be better probably. rTorrent could write the info hash to a directory watched by us.
            # completed = self.proxy.d.get_complete( infoHash );
            if self.transmission.get_torrent(infoHash).doneDate > 0:
                return True
        except Exception:
            logger.exception(
                "Got exception while trying to check torrent's completion status. Info hash: '%s'."
                % infoHash
            )

        return False

    # It doesn't delete the data.
    def DeleteTorrent(self, logger, infoHash):
        try:
            self.transmission.stop_torrent(infoHash)
            self.transmission.remove_torrent(infoHash, delete_data=False)
        except Exception:
            logger.exception(
                "Got exception while trying to delete torrent. Info hash: '%s'."
                % infoHash
            )

    # Transmission doesn't have any problems with this.. so just skip
    def CleanTorrentFile(self, logger, torrentPath):
        # logger.info( "Cleaning torrent file '%s'... nah" % torrentPath )
        pass



import os
import subprocess

from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.Settings import Settings


class ImageMagick:
    # Compresses PNG more if possible.
    # The compression is lossless.
    @staticmethod
    def OptimizePng(logger, sourceImagePath):
        logger.info("Optimizing PNG '%s' with ImageMagick." % (sourceImagePath))

        if not os.path.isfile(sourceImagePath):
            raise PtpUploaderException(
                "Can't read source image '%s' for PNG optimization." % sourceImagePath
            )

        outputImagePath = sourceImagePath + ".optimized.png"
        if os.path.exists(outputImagePath):
            raise PtpUploaderException(
                "Can't optimize PNG because output file '%s' already exists."
                % outputImagePath
            )

        args = [
            Settings.ImageMagickConvertPath,
            sourceImagePath,
            "-define",
            "png:exclude-chunk=gAMA",
            "-define",
            "png:exclude-chunk=cHRM",
            "-quality",
            "99",
            outputImagePath,
        ]
        errorCode = subprocess.call(args)
        if errorCode != 0:
            raise PtpUploaderException(
                "Process execution '%s' returned with error code '%s'."
                % (args, errorCode)
            )

        sourceSize = os.path.getsize(sourceImagePath)
        outputSize = os.path.getsize(outputImagePath)
        gainedBytes = sourceSize - outputSize
        if outputSize > 0 and gainedBytes > 0:
            os.remove(sourceImagePath)
            os.rename(outputImagePath, sourceImagePath)
            logger.info("Optimized PNG is %s bytes smaller." % (gainedBytes))
        else:
            os.remove(outputImagePath)



import re

from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.PtpUploaderException import PtpUploaderException


class MoviePoster:
    # We use The Internet Movie Poster DataBase's embedding code to get the movie poster.
    @staticmethod
    def __GetFromMoviePosterDb(imdbId):
        url = "http://www.movieposterdb.com/embed.inc.php?movie_id=%s" % imdbId
        result = MyGlobals.session.get(url)
        result.raise_for_status()
        response = result.text

        # Response looks like this:
        # document.write('<a target=\"_new\" href=\"http://www.movieposterdb.com/movie/0333766/Garden-State.html\"><img title=\"Garden State\" alt=\"Garden State\" style=\"width: 100px; height: 143px; border: 0px;\" src=\"http://www.movieposterdb.com/posters/08_11/2004/333766/t_333766_b9a6e423.jpg\" /></a>');
        match = re.search(r'src=\\"(.+)\\"', response)
        if match:
            url = match.group(1)
            url = url.replace("/t_", "/l_")  # Change thumbnail to full image.
            return url

        return ""

    @staticmethod
    def Get(logger, imdbId):
        logger.info(
            "Getting movie poster from The Internet Movie Poster DataBase for IMDb id '%s'."
            % imdbId
        )

        # We don't care if this fails. It gives only extra information.
        try:
            return MoviePoster.__GetFromMoviePosterDb(imdbId)
        except Exception:
            logger.exception(
                "Got exception while trying to get poster from The Internet Movie Poster DataBase by IMDb id '%s'."
                % imdbId
            )

        return ""



import os
import logging
import subprocess

from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.Settings import config

logger = logging.getLogger(__name__)


def optimize_png(sourceImagePath: os.PathLike):
    logger.info("Optimizing PNG '%s' with oxipng." % (sourceImagePath))
    if not os.path.isfile(sourceImagePath):
        raise PtpUploaderException(
            "Can't read source image '%s' for PNG optimization." % sourceImagePath
        )
    args = [config.tools.oxipng.path, "-o", "3", "--strip", "all", sourceImagePath]
    proc = subprocess.run(args, capture_output=True, encoding="utf-8")
    if proc.returncode != 0:
        raise PtpUploaderException(
            "Process execution '%s' returned with error code '%s'."
            % (args, proc.returncode)
        )
    logger.info(proc.stderr)



import os

from pyrosimple.util.metafile import Metafile

from PtpUploader.Settings import Settings


def Make(logger, path, torrentPath):
    logger.info("Making torrent from '%s' to '%s'." % (path, torrentPath))

    if os.path.exists(torrentPath):
        # We should be safe to allow the existing torrent to be used,
        # even when/if file selection is re-implemented, all the filesystem
        # manipulation has to be performed before we reach this API.
        # If it changes, at that point we can reset the torrentPath
        # and let it get rebuilt here.

        # Ignore the result of this method, we just want to check that files haven't changed/moved
        metafile = Metafile.from_file(torrentPath)
        metafile.add_fast_resume(path)
        logger.info("Using existing torrent file at '%s'.", torrentPath)
    else:
        logger.info("Making torrent from '%s' to '%s'.", path, torrentPath)
        metafile = Metafile.from_path(
            path,
            Settings.PtpAnnounceUrl,
            created_by="PtpUploader",
            private=True,
            progress=None,
        )
        metafile["info"]["source"] = "PTP"
        metafile.save(torrentPath)



"""
ASGI config for mysite project.

It exposes the ASGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/3.2/howto/deployment/asgi/
"""

import os

from django.core.wsgi import get_wsgi_application


os.environ.setdefault("DJANGO_SETTINGS_MODULE", "PtpUpload.Settings")

application = get_wsgi_application()



from imdb import IMDb


class ImdbInfo:
    def __init__(self):
        self.Title: str = ""
        self.Year: str = ""
        self.PosterUrl: str = ""
        self.Plot: str = ""
        self.IsSeries: bool = False
        self.ImdbRating: str = ""
        self.ImdbVoteCount: str = ""
        self.Raw = None


class Imdb:
    @staticmethod
    def GetInfo(logger, imdbId: str) -> ImdbInfo:
        logger.info("Getting IMDb info for IMDb id '%s'." % imdbId)

        # We don't care if this fails. It gives only extra information.
        imdbInfo = ImdbInfo()
        try:
            ia = IMDb()
            movie = ia.get_movie(imdbId.strip("t"))
            imdbInfo.Raw = movie
            if "title" in movie:
                imdbInfo.Title = movie["title"]
            if "year" in movie:
                imdbInfo.Year = str(movie["year"])
            if "rating" in movie:
                imdbInfo.ImdbRating = movie["rating"]
            if "votes" in movie:
                imdbInfo.ImdbVoteCount = movie["votes"]
            if "full-size cover url" in movie:
                imdbInfo.PosterUrl = movie["full-size cover url"]
            if "plot" in movie:
                imdbInfo.Plot = movie["plot"][0]
        except Exception:
            logger.exception(
                "Got exception while trying to get IMDb info by IMDb id '%s'." % imdbId
            )

        return imdbInfo



# The __init__.py files are required to make Python treat the directories as containing packages
# http://docs.python.org/tutorial/modules.html
from PtpUploader.ImageHost.ImageUploader import ImageUploader


def list_hosts():
    return ["ptpimg", "catbox", "imgbb"]


def upload(logger, imagePath=None, imageUrl=None):
    return ImageUploader.Upload(logger, imagePath, imageUrl)



# Generated by Django 3.2.11 on 2022-01-15 03:38

import datetime
from django.db import migrations, models
from django.utils.timezone import utc


class Migration(migrations.Migration):

    dependencies = [
        ('web', '0014_rename_scheduletimeutc_releaseinfo_scheduletime'),
    ]

    operations = [
        migrations.AlterField(
            model_name='releaseinfo',
            name='ScheduleTime',
            field=models.DateTimeField(default=datetime.datetime(1970, 1, 1, 0, 0, tzinfo=utc), null=True),
        ),
    ]



import logging
import os
import re
import shutil

from pathlib import Path

from PtpUploader import nfo_parser
from PtpUploader.IncludedFileList import IncludedFileList
from PtpUploader.Job.FinishedJobPhase import FinishedJobPhase
from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.release_extractor import parse_directory
from PtpUploader.Settings import Settings, config


logger = logging.getLogger(__name__)


class SourceBase:
    def __init__(self):
        self.MaximumParallelDownloads = 1
        self.Name = None
        self.NameInSettings = None

    def LoadSettingOrDefault(self, key: str):
        if key in self.settings.keys():
            return self.settings.get(key)
        return config.source._default.get(key)

    def LoadSettings(self, _):
        self.settings = config.source.get(self.NameInSettings, {})
        self.AutomaticJobFilter = self.LoadSettingOrDefault("scene")
        # self.Username = settings.GetDefault(self.NameInSettings, "Username", "")
        # self.Password = settings.GetDefault(self.NameInSettings, "Password", "")

        # Do not allow bogus settings.
        maximumParallelDownloads = int(
            self.LoadSettingOrDefault("max_active_downloads")
        )
        if maximumParallelDownloads > 0:
            self.MaximumParallelDownloads = maximumParallelDownloads

        self.AutomaticJobStartDelay = int(self.LoadSettingOrDefault("job_start_delay"))

        self.StopAutomaticJob: bool = self.LoadSettingOrDefault("always_stop_before")
        self.StopAutomaticJobIfThereAreMultipleVideos: bool = self.LoadSettingOrDefault(
            "stop_if_multiple_videos"
        )

    def IsEnabled(self) -> bool:
        return config.source.get(self.NameInSettings) is not None

    def Login(self):
        pass

    def PrepareDownload(self, _, releaseInfo):
        pass

    def ParsePageForExternalCreateJob(self, _, releaseInfo, html):
        pass

    def CheckSynopsis(self, _, releaseInfo):
        # If it exists on PTP then we don't need a synopsis.
        if (not releaseInfo.Synopsis) and (not releaseInfo.PtpId):
            raise PtpUploaderException("Synopsis is not set.")

    def CheckCoverArt(self, _, releaseInfo):
        # If it exists on PTP then we don't need a cover.
        if (not releaseInfo.CoverArtUrl) and (not releaseInfo.PtpId):
            raise PtpUploaderException("Cover art is not set.")

    def DownloadTorrent(self, _, releaseInfo, path):
        pass

    # fileList must be an instance of IncludedFileList.
    def CheckFileList(self, releaseInfo, fileList):
        logger.info("Checking the contents of the release.")

        if releaseInfo.IsDvdImage():
            return

        # Check if the release contains multiple non-ignored videos.
        numberOfVideoFiles = 0
        for file in fileList.Files:
            if file.IsIncluded() and Settings.HasValidVideoExtensionToUpload(file.Name):
                numberOfVideoFiles += 1

        if numberOfVideoFiles > 1:
            raise PtpUploaderException("Release contains multiple video files.")

    # fileList must be an instance of IncludedFileList.
    def DetectSceneReleaseFromFileList(self, releaseInfo, fileList):
        rarRe = re.compile(r".+\.(?:rar|r\d+)$", re.IGNORECASE)
        rarCount = 0

        for file in fileList.Files:
            if rarRe.match(file.Name) is None:
                continue

            # If there are multiple RAR files then it's likely a scene release.
            rarCount += 1
            if rarCount > 1:
                releaseInfo.SetSceneRelease()
                break

    def IsDownloadFinished(self, _, releaseInfo):
        return Settings.GetTorrentClient().IsTorrentFinished(
            logger, releaseInfo.SourceTorrentInfoHash
        )

    def GetCustomUploadPath(self, _, releaseInfo):
        return ""

    def CreateUploadDirectory(self, releaseInfo):
        uploadDirectory = releaseInfo.GetReleaseUploadPath()
        logger.info("Creating upload directory at '%s'." % uploadDirectory)

        if os.path.exists(uploadDirectory):
            raise PtpUploaderException(
                "Upload directory '%s' already exists." % uploadDirectory
            )

        os.makedirs(uploadDirectory)

    def ReadNfo(self, releaseInfo):
        releaseInfo.Nfo = nfo_parser.find_and_read_nfo(
            releaseInfo.GetReleaseDownloadPath()
        )

    def GetIncludedFileList(self, releaseInfo):
        includedFileList = IncludedFileList()

        if os.path.isfile(releaseInfo.SourceTorrentFilePath):
            includedFileList.FromTorrent(releaseInfo.SourceTorrentFilePath)

        return includedFileList

    @staticmethod
    def __DeleteDirectoryIfEmpyOrContainsOnlyEmptyDirectories(path):
        if not os.path.isdir(path):
            return

        for (_, _, fileNames) in os.walk(path):
            for _ in fileNames:
                return

        # Avoid shutil.rmtree because it's scary
        for (root, _, _) in os.walk(path, topdown=False):
            os.rmdir(root)

    def Delete(self, releaseInfo, torrentClient, deleteSourceData, deleteUploadData):
        # Only delete if the release directory has been created by this job.
        # (This is needed because of the releases with the same name. This way deleting the second one won't delete the release directory of the first.)
        if not releaseInfo.IsJobPhaseFinished(
            FinishedJobPhase.Download_CreateReleaseDirectory
        ):
            return

        if (
            deleteSourceData or deleteUploadData
        ) and not config.uploader.allow_recursive_delete:
            raise PtpUploaderException(
                "Recursive delete requested but functionality is disabled by uploader.allow_recursive_delete setting"
            )

        if deleteSourceData:
            # Delete the source torrent file.
            if releaseInfo.SourceTorrentFilePath and os.path.isfile(
                releaseInfo.SourceTorrentFilePath
            ):
                os.remove(releaseInfo.SourceTorrentFilePath)

            # Delete the source torrent from the torrent client.
            if len(releaseInfo.SourceTorrentInfoHash) > 0:
                torrentClient.DeleteTorrent(logger, releaseInfo.SourceTorrentInfoHash)

            # Delete the data of the source torrent.
            if os.path.isdir(releaseInfo.GetReleaseDownloadPath()):
                shutil.rmtree(releaseInfo.GetReleaseDownloadPath())

        if deleteUploadData:
            # Delete the uploaded torrent file.
            if releaseInfo.UploadTorrentFilePath and os.path.isfile(
                releaseInfo.UploadTorrentFilePath
            ):
                os.remove(releaseInfo.UploadTorrentFilePath)

            # Delete the uploaded torrent from the torrent client.
            if len(releaseInfo.UploadTorrentInfoHash) > 0:
                torrentClient.DeleteTorrent(logger, releaseInfo.UploadTorrentInfoHash)

            # Delete the data of the uploaded torrent.
            if os.path.isdir(releaseInfo.GetReleaseUploadPath()):
                shutil.rmtree(releaseInfo.GetReleaseUploadPath())

        if deleteSourceData and deleteUploadData:
            SourceBase.__DeleteDirectoryIfEmpyOrContainsOnlyEmptyDirectories(
                releaseInfo.GetReleaseRootPath()
            )

        log = Path(Settings.GetJobLogPath(), str(releaseInfo.Id))
        if log.is_file():
            log.unlink()

    def GetTemporaryFolderForImagesAndTorrent(self, releaseInfo):
        return releaseInfo.GetReleaseRootPath()

    def IsSingleFileTorrentNeedsDirectory(self, _) -> bool:
        return True

    def IncludeReleaseNameInReleaseDescription(self):
        return True

    def GetIdFromUrl(self, url: str) -> str:
        return ""

    def GetUrlFromId(self, id: str) -> str:
        return ""

    def GetIdFromAutodlIrssiUrl(self, url: str) -> str:
        return ""



# Generated by Django 3.2.5 on 2021-10-29 21:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('web', '0008_alter_releaseinfo_subtitles'),
    ]

    operations = [
        migrations.AlterField(
            model_name='releaseinfo',
            name='Trumpable',
            field=models.JSONField(blank=True, default=list),
        ),
    ]



import html
import logging
import os
import re

import unidecode

from guessit import guessit

from PtpUploader.Helper import (
    GetSizeFromText,
    RemoveDisallowedCharactersFromPath,
    ValidateTorrentFile,
)
from PtpUploader.InformationSource.Imdb import Imdb
from PtpUploader.Job.JobRunningState import JobRunningState
from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.Source.SourceBase import SourceBase


logger = logging.getLogger(__name__)


class Cinemageddon(SourceBase):
    def __init__(self):
        SourceBase.__init__(self)

        self.Name = "cg"
        self.NameInSettings = "Cinemageddon"

    def LoadSettings(self, _):
        super().LoadSettings(_)
        self.Username = self.settings.username
        self.Password = self.settings.password

    def Login(self):
        logger.info("Logging in to Cinemageddon.")

        if "cinemageddon.net" not in MyGlobals.session.cookies.list_domains():
            postData = {"username": self.Username, "password": self.Password}
            result = MyGlobals.session.post(
                "https://cinemageddon.net/takelogin.php", data=postData
            )
            result.raise_for_status()
            self.__CheckIfLoggedInFromResponse(result.content)
            MyGlobals.SaveCookies()
        else:
            logger.debug("Re-using Cinemageddon cookies.")

    def __CheckIfLoggedInFromResponse(self, response: bytes):
        if b'action="takelogin.php"' in response or b"Login failed!" in response:
            raise PtpUploaderException(
                f"Looks like you are not logged in to Cinemageddon. Probably due to the bad user name or password in settings. Response: {response.decode()}"
            )

    def __ParsePage(
        self, _, releaseInfo, raw_html: bytes, parseForExternalCreateJob=False
    ):
        # Make sure we only get information from the description and not from the comments.
        descriptionEndIndex = raw_html.find(b'<p><a name="startcomments"></a></p>')
        if descriptionEndIndex == -1:
            raise PtpUploaderException(
                JobRunningState.Ignored_MissingInfo,
                "Description can't found on torrent page. Probably the layout of the site has changed.",
            )

        description: bytes = raw_html[:descriptionEndIndex]

        # We will use the torrent's name as release name.
        if not parseForExternalCreateJob:
            matches = re.search(
                rb'href="download.php\?id=(\d+)&name=.+">(.+)\.torrent</a>', description
            )
            if matches is None:
                raise PtpUploaderException(
                    JobRunningState.Ignored_MissingInfo,
                    "Can't get release name from torrent page.",
                )

            releaseInfo.ReleaseName = html.unescape(matches.group(2).decode())

        # Get source and format type
        sourceType = ""
        formatType = ""
        if not releaseInfo.Source or not releaseInfo.Codec:
            matches = None
            if parseForExternalCreateJob:
                matches = re.search(
                    rb'torrent details for "(.+) \[(\d+)/(.+)/(.+)\]"', description
                )
            else:
                matches = re.search(
                    rb"torrent details for &quot;(.+) \[(\d+)/(.+)/(.+)\]&quot;",
                    description,
                )

            if matches is None:
                raise PtpUploaderException(
                    JobRunningState.Ignored_MissingInfo,
                    "Can't get release source and format type from torrent page.",
                )

            sourceType = matches.group(3).decode()
            formatType = matches.group(4).decode()

            if "/" in sourceType and not releaseInfo.ResolutionType:
                sourceType, _, resolutionType = sourceType.partition("/")
                resolutionType.strip("p")
                if resolutionType in ["576", "720", "1080"]:
                    releaseInfo.ResolutionType = resolutionType + "p"
                else:
                    releaseInfo.ResolutionType = "Other"

        # Get IMDb id.
        if (not releaseInfo.ImdbId) and (not releaseInfo.PtpId):
            match = re.search(
                r'<span id="torrent_imdb">(.*?)</span>',
                description.decode(errors="ignore"),
            )
            if not match:
                raise PtpUploaderException(
                    JobRunningState.Ignored_MissingInfo,
                    "IMDb container can't be found on torrent page.",
                )
            imdbs = match.group(1).replace("t", "").split(" ")
            if not imdbs:
                raise PtpUploaderException(
                    JobRunningState.Ignored_MissingInfo,
                    "IMDb id can't be found on torrent page.",
                )
            if len(imdbs) > 1:
                raise PtpUploaderException(
                    JobRunningState.Ignored_MissingInfo,
                    "Multiple IMDb IDs found on page.",
                )

            releaseInfo.ImdbId = imdbs[0]

        # Get size.
        # Two possible formats:
        # <tr><td class="rowhead" valign="top" align="right">Size</td><td valign="top" align="left">1.46 GB (1,570,628,119 bytes)</td></tr>
        # <tr><td class="rowhead" valign="top" align="right">Size</td><td valign="top" align=left>1.46 GB (1,570,628,119 bytes)</td></tr>
        matches = re.search(
            rb"""<tr><td class="rowhead" valign="top" align="right">Size</td><td valign="top" align="?left"?>.+ \((.+ bytes)\)</td></tr>""",
            description,
        )
        if matches is None:
            releaseInfo.logger().warning("Size not found on torrent page.")
        else:
            size: bytes = matches.group(1)
            releaseInfo.Size = GetSizeFromText(size.decode())

        # Ignore XXX releases.
        if description.find(b'>Type</td><td valign="top" align=left>XXX<') != -1:
            raise PtpUploaderException(
                JobRunningState.Ignored_Forbidden, "Marked as XXX."
            )

        self.__MapSourceAndFormatToPtp(releaseInfo, sourceType, formatType, raw_html)

        # Make sure that this is not a wrongly categorized DVDR.
        if (not releaseInfo.IsDvdImage()) and (
            re.search(rb"\.vob</td>", description, re.IGNORECASE)
            or re.search(rb"\.iso</td>", description, re.IGNORECASE)
        ):
            raise PtpUploaderException(
                JobRunningState.Ignored_NotSupported, "Wrongly categorized DVDR."
            )

    def __DownloadNfo(self, _, releaseInfo):
        url = (
            "https://cinemageddon.net/details.php?id=%s&filelist=1"
            % releaseInfo.AnnouncementId
        )
        releaseInfo.logger().info("Collecting info from torrent page '%s'.", url)

        result = MyGlobals.session.get(url)
        result.raise_for_status()
        response = result.content
        self.__CheckIfLoggedInFromResponse(response)

        self.__ParsePage(releaseInfo.logger(), releaseInfo, response)

    def __MapSourceAndFormatToPtp(
        self, releaseInfo, sourceType, formatType, raw_html: bytes
    ):
        sourceType = sourceType.lower()
        formatType = formatType.lower()

        if releaseInfo.Source:
            releaseInfo.logger().info(
                "Source '%s' is already set, not getting from the torrent page.",
                releaseInfo.Source,
            )
        elif sourceType in ["dvdrip", "dvd-r"]:
            releaseInfo.Source = "DVD"
        elif sourceType == "vhsrip":
            releaseInfo.Source = "VHS"
        elif sourceType == "tvrip":
            releaseInfo.Source = "TV"
        elif sourceType in ["webrip", "web-dl"]:
            releaseInfo.Source = "WEB"
        elif sourceType == "bdrip":  # brrips exist but aren't allowed on PTP
            releaseInfo.Source = "Blu-ray"
        else:
            raise PtpUploaderException(
                JobRunningState.Ignored_NotSupported,
                "Unsupported source type '%s'." % sourceType,
            )

        if releaseInfo.Codec:
            releaseInfo.logger().info(
                "Codec '%s' is already set, not getting from the torrent page.",
                releaseInfo.Codec,
            )
        elif formatType == "x264":
            releaseInfo.Codec = "x264"
        elif formatType == "xvid":
            releaseInfo.Codec = "XviD"
        elif formatType == "divx":
            releaseInfo.Codec = "DivX"
        elif formatType == "dvd-r":
            if releaseInfo.Size > 4700000000:
                releaseInfo.Codec = "DVD9"
            else:
                releaseInfo.Codec = "DVD5"
        else:
            raise PtpUploaderException(
                JobRunningState.Ignored_NotSupported,
                "Unsupported format type '%s'." % formatType,
            )

        # Adding BDrip support would be problematic because there is no easy way to decide if it is HD or SD.
        # Maybe we could use the resolution and file size. But what about the oversized and upscaled releases?

        if releaseInfo.ResolutionType:
            releaseInfo.logger().info(
                "Resolution type '%s' is already set, not getting from the torrent page.",
                releaseInfo.ResolutionType,
            )
        elif releaseInfo.IsDvdImage():
            if re.search(rb"Standard +: NTSC", raw_html):
                releaseInfo.ResolutionType = "NTSC"
            elif re.search(rb"Standard +: PAL", raw_html):
                releaseInfo.ResolutionType = "PAL"
            else:
                releaseInfo.logger().info("DVD detected but could not find resolution")
        elif releaseInfo.Source not in ["Blu-ray", "WEB"]:
            releaseInfo.ResolutionType = "Other"
        else:
            releaseInfo.logger().info("Could not detect resolution")

        if releaseInfo.IsDvdImage() and not releaseInfo.Container:
            if re.search(rb"\.vob</td>", raw_html, re.IGNORECASE):
                releaseInfo.Container = "VOB IFO"
            elif re.search(rb"\.iso</td>", raw_html, re.IGNORECASE):
                releaseInfo.Container = "ISO"
            else:
                releaseInfo.logger().info("DVD detected but could not find container")

    def PrepareDownload(self, _, releaseInfo):
        if releaseInfo.IsUserCreatedJob():
            self.__DownloadNfo(releaseInfo.logger(), releaseInfo)
        else:
            # TODO: add filtering support for Cinemageddon
            # In case of automatic announcement we have to check the release name if it is valid.
            # We know the release name from the announcement, so we can filter it without downloading anything (yet) from the source.
            # if not ReleaseFilter.IsValidReleaseName( releaseInfo.ReleaseName ):
            #    logger.info( "Ignoring release '%s' because of its name." % releaseInfo.ReleaseName )
            #    return None
            self.__DownloadNfo(releaseInfo.logger(), releaseInfo)

    def ParsePageForExternalCreateJob(self, _, releaseInfo, html):
        self.__ParsePage(
            releaseInfo.logger(), releaseInfo, html, parseForExternalCreateJob=True
        )

    def DownloadTorrent(self, _, releaseInfo, path):
        url = "https://cinemageddon.net/download.php?id=%s" % releaseInfo.AnnouncementId
        releaseInfo.logger().info(
            "Downloading torrent file from '%s' to '%s'.", url, path
        )

        result = MyGlobals.session.get(url)
        result.raise_for_status()
        response = result.content
        self.__CheckIfLoggedInFromResponse(response)

        # The number of maximum simultaneous downloads is limited on Cinemageddon.
        if response.find(b"<h2>Max Torrents Reached</h2>") != -1:
            raise PtpUploaderException("Maximum torrents reached on CG.")

        with open(path, "wb") as fh:
            fh.write(response)

        ValidateTorrentFile(path)

    # Because some of the releases on CG do not contain the full name of the movie, we have to rename them because of the uploading rules on PTP.
    # The new name will be formatted like this: Movie Name Year
    def GetCustomUploadPath(self, _, releaseInfo) -> str:
        # TODO: if the user forced a release name, then let it upload by that name.
        if releaseInfo.ImdbId == "0":
            raise PtpUploaderException(
                "Uploading from CG with zero IMDb ID is not yet supported."
            )

        # If the movie already exists on PTP then the IMDb info is not populated in ReleaseInfo.
        if not releaseInfo.InternationalTitle or not releaseInfo.Year:
            imdbInfo = Imdb.GetInfo(logger, releaseInfo.ImdbId)
            # If the release name matches IMDb, preserve the name
            guess = guessit(releaseInfo.ReleaseName, {"enforce_list": True})
            if "title" in guess and imdbInfo.Title in guess["title"]:
                return releaseInfo.GetReleaseUploadPath()
            if not releaseInfo.InternationalTitle:
                releaseInfo.InternationalTitle = imdbInfo.Title
            if not releaseInfo.Year:
                releaseInfo.Year = imdbInfo.Year
            releaseInfo.save()

        if not releaseInfo.InternationalTitle:
            print(imdbInfo.Raw)
            raise PtpUploaderException("No title found, cannot rename.")

        if not releaseInfo.Year:
            print(imdbInfo.Raw)
            raise PtpUploaderException("No year found, cannot rename.")

        if len(releaseInfo.Year) <= 0:
            raise PtpUploaderException("Can't rename release because year is empty.")

        name = "{} ({}) {} {}".format(
            releaseInfo.InternationalTitle,
            releaseInfo.Year,
            releaseInfo.Source,
            releaseInfo.Codec,
        )
        name = RemoveDisallowedCharactersFromPath(name)

        releaseInfo.logger().info(
            "Upload directory will be named '%s' instead of '%s'.",
            name,
            releaseInfo.ReleaseName,
        )

        newUploadPath = releaseInfo.GetReleaseUploadPath()
        newUploadPath = os.path.dirname(newUploadPath)
        newUploadPath = os.path.join(newUploadPath, unidecode.unidecode(name))
        return newUploadPath

    def IncludeReleaseNameInReleaseDescription(self):
        return False

    def GetIdFromUrl(self, url):
        result = re.match(r".*cinemageddon\.net/details.php\?id=(\d+).*", url)
        if result is None:
            return ""
        return result.group(1)

    def GetUrlFromId(self, id):
        return "https://cinemageddon.net/details.php?id=" + id



# Generated by Django 3.2.9 on 2022-01-08 21:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('web', '0013_auto_20211104_0344'),
    ]

    operations = [
        migrations.RenameField(
            model_name='releaseinfo',
            old_name='ScheduleTimeUtc',
            new_name='ScheduleTime',
        ),
    ]



import logging

from PtpUploader.Settings import config


class ImageSite:
    def __init__(self, logger=None):
        if logger is None:
            self.logger = logging.getLogger(__name__)
        else:
            self.logger = logger
        self.config = config.image_host[self.name]

    def upload_url(self, url: str):
        raise NotImplementedError

    def upload_path(self, path: str):
        raise NotImplementedError



"""
Replaces the artisanal (but impressive) python 2 threading system
This directly handles:
- Loading announcement files
- Scanning the DB for work
- Watching the client for pending downloads
All of these actions will happen on a schedule, but the thread can be woken up immediately by an Event.

For WorkerBase actions, they get launched into a ThreadPoolExecutor, with an event flag
to allow for interrupting the work phases. This allows this class to check the status of
any active futures when deciding how to handle start/stop requests.

However, there is no direct locking of resources, as we can use the DB as more flexible thread-safe state
holder.

It's called the JobSupervisor because supervisors are better than managers.
"""

import logging
import queue
import threading
import traceback

from concurrent import futures
from typing import Dict, List

from django.db.models import Q  # type: ignore
from django.utils import timezone  # type: ignore
from pyrosimple.util.rpc import HashNotFound

from PtpUploader.Job import LoadFile
from PtpUploader.Job.CheckAnnouncement import CheckAnnouncement
from PtpUploader.Job.Delete import Delete
from PtpUploader.Job.Upload import Upload
from PtpUploader.PtpUploaderMessage import *
from PtpUploader.ReleaseInfo import ReleaseInfo
from PtpUploader.Settings import config


logger = logging.getLogger(__name__)


class JobSupervisor(threading.Thread):
    def __init__(self):
        super().__init__()
        self.futures: Dict[str, List[threading.Event, futures.Future]] = {}
        self.message_queue: queue.SimpleQueue = queue.SimpleQueue()
        self.stop_requested: threading.Event = threading.Event()
        self.pool: futures.ThreadPoolExecutor = futures.ThreadPoolExecutor(
            max_workers=config.workers.threads
        )

    def __repr__(self):
        running = []
        done = []
        waiting = []
        for key, val in self.futures.items():
            _, future = val
            if future.running():
                running.append(key)
            elif future.done():
                done.append(key)
            else:
                waiting.append(key)
        return f"running: {running}, waiting: {waiting}, done: {done}"

    def check_pending_downloads(self):
        for release in ReleaseInfo.objects.filter(
            JobRunningState=ReleaseInfo.JobState.InDownload
        ):
            try:
                release.AnnouncementSource.IsDownloadFinished(logger, release)
            except HashNotFound as e:
                release.ErrorMessage = str(e)
                release.JobRunningState = ReleaseInfo.JobState.Failed
                release.save()
                continue
            if (
                release.Id not in self.futures
                and release.AnnouncementSource.IsDownloadFinished(logger, release)
            ):
                logger.info("Launching upload job for %s", release.Id)
                worker_stop_flag = threading.Event()
                worker = Upload(release_id=release.Id, stop_requested=worker_stop_flag)
                self.futures[release.Id] = [
                    worker_stop_flag,
                    self.pool.submit(worker.Work),
                ]

    def load_announcements(self):
        LoadFile.scan_dir()

    def add_message(self, message):
        if isinstance(message, PtpUploaderMessageBase):
            self.message_queue.put(message)
        else:
            logger.warning("Unknown message '%s'", message)

    def delete_job(self, r_id, mode):
        if r_id in self.futures:
            return  # Don't muck with an active job
        worker_stop_flag = threading.Event()
        worker = Delete(release_id=r_id, mode=mode, stop_requested=worker_stop_flag)
        self.futures[r_id] = [
            worker_stop_flag,
            self.pool.submit(worker.Work),
        ]

    def scan_db(self):
        """Find releases pending work by their DB status"""
        for release in ReleaseInfo.objects.filter(
            Q(JobRunningState=ReleaseInfo.JobState.WaitingForStart)
            | (
                Q(ScheduleTime__lte=timezone.now())
                & Q(JobRunningState=ReleaseInfo.JobState.Scheduled)
            )
        ):
            if release.Id not in self.futures:
                logger.info("Launching check job for %s", release.Id)
                worker_stop_flag = threading.Event()
                worker = CheckAnnouncement(
                    release_id=release.Id, stop_requested=worker_stop_flag
                )
                self.futures[release.Id] = [
                    worker_stop_flag,
                    self.pool.submit(worker.Work),
                ]

    def process_pending(self):
        self.check_pending_downloads()
        self.load_announcements()
        self.scan_db()

    def stop_future(self, release_id):
        release = ReleaseInfo.objects.get(Id=release_id)
        if release.Id in self.futures:
            pass
        elif release.JobRunningState in [
            ReleaseInfo.JobState.InDownload,
            ReleaseInfo.JobState.Scheduled,
        ]:
            release.JobRunningState = ReleaseInfo.JobState.Paused
            release.save()

    def reap_finished(self):
        for key, val in list(self.futures.items()):
            _, result = val
            if result.done():
                if result.exception():
                    logger.info(  # Exceptions are used as messengers, hence info
                        "Job %s finished with exception '%s'", key, result.exception()
                    )
                del self.futures[key]

    def work(self):
        if self.futures.keys():
            logger.info(repr(self))
        try:
            message = self.message_queue.get(timeout=3)
            if isinstance(message, PtpUploaderMessageStopJob):
                self.stop_future(message.ReleaseInfoId)
            elif isinstance(message, PtpUploaderMessageStartJob):
                pass  # Just wake up the thread to scan the db
            elif isinstance(message, PtpUploaderMessageDeleteJob):
                self.delete_job(message.ReleaseInfoId, message.mode)
            elif isinstance(message, PtpUploaderMessageQuit):
                self.stop_requested.set()
        except queue.Empty:
            pass

        if self.stop_requested.is_set():
            self.reap_finished()
            self.cleanup_futures()
            return True
        self.reap_finished()
        self.process_pending()
        return None

    def run(self):
        logger.info("Starting supervisors")
        while True:
            try:
                if self.work() is not None:
                    break
            except (KeyboardInterrupt, SystemExit):
                logger.info("Received system interrupt")
                self.add_message(PtpUploaderMessageQuit())
            except Exception as e:
                print(traceback.print_exc())
                logger.info("Received exception %s, attempting to continue", e)

    def cleanup_futures(self):
        pass



import requests

from PtpUploader.ImageHost.Base import ImageSite
from PtpUploader.PtpUploaderException import PtpUploaderException


class ImgBB(ImageSite):
    def __init__(self, logger=None):
        self.name = "imgbb"
        super().__init__(logger)
        if not self.config.api_key:
            raise PtpUploaderException("imgbb API key is not set")

    def upload_url(self, url: str):
        return self.upload({"image": url}, {})

    def upload_path(self, path: str):
        with open(path, "rb") as imageHandle:
            return self.upload({}, {"image": imageHandle})

    def upload(self, data, files):
        endpoint = f"https://api.imgbb.com/1/upload?key={self.config.api_key}"
        response = requests.post(endpoint, data=data, files=files)
        response.raise_for_status()
        try:
            rjson = response.json()
            return rjson["data"]["url"]
        except (ValueError, KeyError):
            self.logger.exception(
                "Got an exception while loading JSON response from imgbb. Response: '{}'.".format(
                    response
                )
            )
            raise



import html
import logging
import os
import re
import xml.etree.ElementTree as ET

from urllib.parse import urlparse

import requests

from PtpUploader import release_extractor
from PtpUploader.IncludedFileList import IncludedFileList
from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.ReleaseNameParser import ReleaseNameParser
from PtpUploader.Settings import Settings
from PtpUploader.Source.SourceBase import SourceBase


logger = logging.getLogger(__name__)


class Prowlarr(SourceBase):
    def __init__(self):
        super().__init__()
        self.MaximumParallelDownloads = 1
        self.Name = "prowlarr"
        self.NameInSettings = "Prowlarr"

    def LoadSettings(self, _):
        super().LoadSettings(_)
        self.ApiKey = self.settings.api_key
        self.Url = self.settings.url
        self.loaded_indexers = {}

    def Login(self):
        logger.info("Logging into %s", self.Name)
        self.session = requests.Session()
        self.session.headers.update({"X-Api-Key": self.ApiKey})
        r = self.session.get(self.Url + "/api/v1/indexer").json()
        for t in r:
            self.loaded_indexers[t["name"]] = t
        logger.info(
            "Loaded indexers from prowlarr: %s", list(self.loaded_indexers.keys())
        )

    def PrepareDownload(self, _, releaseInfo):
        logger.info("Processing '%s' with prowlarr", releaseInfo.AnnouncementId)
        if not releaseInfo.ImdbId:
            return
        match = self.match_imdb(releaseInfo)
        if match is None:
            logger.warning("Could not find release info in prowlarr")
            return
        for field in match:
            if field.tag == "title" and not releaseInfo.ReleaseName:
                releaseInfo.ReleaseName = field.text
            if field.tag == "size" and not releaseInfo.Size:
                releaseInfo.Size = field.text
        releaseNameParser = ReleaseNameParser(releaseInfo.ReleaseName)
        releaseNameParser.GetSourceAndFormat(releaseInfo)
        if releaseNameParser.Scene:
            releaseInfo.SetSceneRelease()

    def match_imdb(self, releaseInfo):
        indexer = self.get_indexer(releaseInfo)
        response = self.session.get(
            f"{self.Url}/api/v1/indexer/{indexer['id']}/newznab",
            params={"t": "movie", "imdbid": "tt" + str(releaseInfo.ImdbId)},
        )
        for i in ET.fromstring(response.text)[0].findall("item"):
            for field in i:
                if (
                    field.tag in ["guid", "comments"]
                    and field.text == releaseInfo.AnnouncementId
                ):
                    return i
        return None

    def get_indexer(self, release):
        o = urlparse(release.AnnouncementId)
        for t in self.loaded_indexers.values():
            for u in t["indexerUrls"]:
                iu = urlparse(u)
                if iu.netloc == o.netloc:
                    return t
        return None

    def DownloadTorrent(self, _, releaseInfo, path):
        match = self.match_imdb(releaseInfo)
        link = None
        for field in match:
            if field.tag == "link":
                link = field.text
                break
        if link is None:
            raise PtpUploaderException("No download link found in prowlarr")
        with open(path, "wb") as fh:
            fh.write(self.session.get(html.unescape(link)).content)

    # fileList must be an instance of IncludedFileList.
    def CheckFileList(self, releaseInfo, fileList):
        logger.info("Checking the contents of the release.")

        if releaseInfo.IsDvdImage():
            return

        # Check if the release contains multiple non-ignored videos.
        numberOfVideoFiles = 0
        for file in fileList.Files:
            if file.IsIncluded() and Settings.HasValidVideoExtensionToUpload(file.Name):
                numberOfVideoFiles += 1

        if numberOfVideoFiles > 1:
            raise PtpUploaderException("Release contains multiple video files.")

    # fileList must be an instance of IncludedFileList.
    def DetectSceneReleaseFromFileList(self, releaseInfo, fileList):
        rarRe = re.compile(r".+\.(?:rar|r\d+)$", re.IGNORECASE)
        rarCount = 0

        for file in fileList.Files:
            if rarRe.match(file.Name) is None:
                continue

            # If there are multiple RAR files then it's likely a scene release.
            rarCount += 1
            if rarCount > 1:
                releaseInfo.SetSceneRelease()
                break

    def GetIncludedFileList(self, releaseInfo):
        includedFileList = IncludedFileList()

        if os.path.isfile(releaseInfo.SourceTorrentFilePath):
            includedFileList.FromTorrent(releaseInfo.SourceTorrentFilePath)

        return includedFileList

    def GetTemporaryFolderForImagesAndTorrent(self, releaseInfo):
        return releaseInfo.GetReleaseRootPath()

    def IsSingleFileTorrentNeedsDirectory(self, releaseInfo) -> bool:
        return True

    def IncludeReleaseNameInReleaseDescription(self):
        return True

    def GetIdFromUrl(self, url: str) -> str:
        o = urlparse(url)
        for t in self.loaded_indexers.values():
            for u in t["indexerUrls"]:
                iu = urlparse(u)
                if iu.netloc == o.netloc:
                    return url
        return ""

    # Unique situation, the ID is a full URL, since that's what prowlarr's
    # newznab API uses as a guid
    def GetUrlFromId(self, id: str) -> str:
        return id



import bencode

from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.ReleaseNameParser import ReleaseNameParser
from PtpUploader.Source.SourceBase import SourceBase


class Torrent(SourceBase):
    def __init__(self):
        SourceBase.__init__(self)

        self.Name = "torrent"
        self.NameInSettings = "TorrentFileSource"

    def IsEnabled(self):
        return True

    def PrepareDownload(self, logger, releaseInfo):
        # TODO: support for uploads from torrent without specifying IMDb id and reading it from NFO. (We only get IMDb id when the download is finished.)

        # TODO: support for new movies without IMDB id
        if (not releaseInfo.ImdbId) and (not releaseInfo.PtpId):
            raise PtpUploaderException("Doesn't contain IMDb ID.")
        if not releaseInfo.ReleaseName:
            with open(releaseInfo.SourceTorrentFilePath, "rb") as fh:
                meta = bencode.decode(fh.read())
                releaseInfo.ReleaseName = meta["info"]["name"]
        releaseNameParser = ReleaseNameParser(releaseInfo.ReleaseName)
        releaseNameParser.GetSourceAndFormat(releaseInfo)
        if releaseNameParser.Scene:
            releaseInfo.SetSceneRelease()



class JobRunningState:
    WaitingForStart = 0
    InProgress = 1
    Paused = 2
    Finished = 3
    Failed = 4
    Ignored = 5
    Ignored_AlreadyExists = 6
    Ignored_Forbidden = 7
    Ignored_MissingInfo = 8
    Ignored_NotSupported = 9
    DownloadedAlreadyExists = 10
    Scheduled = 11

    @staticmethod
    def ToText(state):
        if state == JobRunningState.WaitingForStart:
            return "Waiting for start"
        elif state == JobRunningState.InProgress:
            return "In progress"
        elif state == JobRunningState.Paused:
            return "Paused"
        elif state == JobRunningState.Finished:
            return "Finished"
        elif state == JobRunningState.Failed:
            return "Failed"
        elif state == JobRunningState.Ignored:
            return "Ignored"
        elif state == JobRunningState.Ignored_AlreadyExists:
            return "Ignored, already exists"
        elif state == JobRunningState.Ignored_Forbidden:
            return "Ignored, forbidden"
        elif state == JobRunningState.Ignored_MissingInfo:
            return "Ignored, missing info"
        elif state == JobRunningState.Ignored_NotSupported:
            return "Ignored, not supported"
        elif state == JobRunningState.DownloadedAlreadyExists:
            return "Downloaded, already exists"
        elif state == JobRunningState.Scheduled:
            return "Scheduled"
        else:
            return "Unknown"



import logging
import re
import shlex
import subprocess

from pathlib import Path
from tempfile import TemporaryDirectory
from typing import List, Optional, Tuple

from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.Settings import config


logger = logging.getLogger(__name__)


def bdinfo_cmd() -> List[str]:
    if config.tools.bdinfo.path:
        return [
            str(Path(p).expanduser()) for p in shlex.split(config.tools.bdinfo.path)
        ]
    else:
        raise PtpUploaderException("BDInfo path not set")


def run(path: Path, mpls=None, extra_args=None) -> str:
    if mpls is None:
        mpls, _ = get_longest_playlist(path)
    logger.info(f"Building bdinfo for path '{path}' and playlist '{mpls}'")
    with TemporaryDirectory() as tempdir:
        args: List[str] = bdinfo_cmd() + [str(path), tempdir, "-m", mpls]
        if extra_args:
            args.append(extra_args)
        subprocess.run(args, check=True)
        for child in Path(tempdir).glob("*"):
            with child.open("r") as fh:
                text = fh.read()
            match = re.search(r"(DISC INFO:.*)FILES:", text, flags=re.M | re.S)
            if match:
                return match.group(1)
    raise PtpUploaderException(f"Could not find BDInfo output for path '{path}'")


def get_longest_playlist(path: Path) -> Tuple[str, int]:
    logger.info(f"Scanning '{path}' for playlists")
    proc = subprocess.run(
        bdinfo_cmd() + [str(path), "-l"], check=True, capture_output=True
    )
    longest_mpls: Optional[str] = None
    longest_len: int = 0
    for line in proc.stdout.decode().split("\n"):
        if ".MPLS" in line:
            length_str = line[26:34].split(":")
            length_sec = sum(
                int(x[1]) * (60 ** (2 - x[0])) for x in enumerate(length_str)
            )
            if length_sec > longest_len:
                longest_len = length_sec
                longest_mpls = line.split(" ")[9]
    if longest_mpls:
        return longest_mpls, longest_len
    else:
        logger.error("Could not parse output: %s", proc.stdout.decode())
        raise PtpUploaderException(f"Could not find playlist from path '{path}'")



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['PtpUploader',
 'PtpUploader.ImageHost',
 'PtpUploader.InformationSource',
 'PtpUploader.Job',
 'PtpUploader.Source',
 'PtpUploader.Tool',
 'PtpUploader.web',
 'PtpUploader.web.management.commands',
 'PtpUploader.web.migrations']

package_data = \
{'': ['*'],
 'PtpUploader.web': ['static/*',
                     'static/script/*',
                     'static/skin-win8/*',
                     'static/source_icon/*',
                     'templates/*']}

install_requires = \
['Django>=3.2.8,<4.0.0',
 'Unidecode>=1.3.2,<2.0.0',
 'Werkzeug>=2.0.2,<3.0.0',
 'cinemagoer>=2022.1.26,<2023.0.0',
 'dynaconf>=3.1.7,<4.0.0',
 'guessit>=3.3.1,<4.0.0',
 'pyrosimple>=2.7.0,<3.0.0',
 'rarfile>=4.0,<5.0',
 'requests>=2.26.0,<3.0.0']

entry_points = \
{'console_scripts': ['PtpUploader = PtpUploader.manage:run',
                     'ReleaseInfoMaker = PtpUploader.ReleaseInfoMaker:run']}

setup_kwargs = {
    'name': 'ptpuploader',
    'version': '0.11.0',
    'description': 'A small uploader for a mildly popular movie site',
    'long_description': 'None',
    'author': 'kannibalox',
    'author_email': 'kannibalox@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/kannibalox/PtpUploader',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>3.7.2,<4.0',
}


setup(**setup_kwargs)



# Generated by Django 3.2.5 on 2021-10-29 01:35

import re

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("web", "0004_auto_20211028_2254"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="releaseinfo",
            name="Flags",
        ),
        migrations.AddField(
            model_name="releaseinfo",
            name="Tags",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="releaseinfo",
            name="Subtitles",
            field=models.TextField(
                blank=True,
                default="",
                validators=[
                    django.core.validators.RegexValidator(
                        re.compile("^\\d+(?:,\\d+)*\\Z"),
                        code="invalid",
                        message="Enter only digits separated by commas.",
                    )
                ],
            ),
        ),
    ]



class JobStartMode:
    Automatic = 0
    Manual = 1

    """
	If this is set then:
	- there is no duplicate checking
	- there is no adult tag checking
	- series is not rejected
	- user specified container overrides the one returned by MediaInfo (even if they are different)
	- user specified codec overrides the one returned by MediaInfo (even if they are different)
	- user specified resolution overrides the one returned by MediaInfo (even if they are different)
	"""
    ManualForced = 2



import datetime
import logging
import os
import subprocess
import threading

from pathlib import Path
from typing import List

from django.template import engines

from PtpUploader import ImageHost, Ptp, ptp_subtitle
from PtpUploader.Helper import GetIdxSubtitleLanguages, TimeDifferenceToText
from PtpUploader.Job.FinishedJobPhase import FinishedJobPhase
from PtpUploader.Job.JobRunningState import JobRunningState
from PtpUploader.Job.JobStartMode import JobStartMode
from PtpUploader.Job.WorkerBase import WorkerBase
from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.PtpUploaderException import *
from PtpUploader.release_extractor import extract_release, parse_directory
from PtpUploader.ReleaseDescriptionFormatter import ReleaseDescriptionFormatter
from PtpUploader.ReleaseInfo import ReleaseInfo
from PtpUploader.Settings import Settings
from PtpUploader.Tool import MediaInfo, Mktor


logger = logging.getLogger(__name__)


class Upload(WorkerBase):
    def __init__(self, release_id: int, stop_requested: threading.Event):
        super().__init__(release_id, stop_requested)
        self.Phases = [
            self.__SetInProgress,
            self.__StopAutomaticJobBeforeExtracting,
            self.__StopAutomaticJobIfThereAreMultipleVideosBeforeExtracting,
            self.__CreateUploadPath,
            self.__MakeIncludedFileList,
            self.__ExtractRelease,
            self.__MakeReleaseDescription,
            self.__DetectSubtitles,
            self.__MakeTorrent,
            self.__CheckIfExistsOnPtp,
            self.__CheckSynopsis,
            self.__CheckCoverArt,
            self.__RehostPoster,
            self.__StopBeforeUploading,
            self.__StartTorrent,
            self.__UploadMovie,
            self.__ExecuteCommandOnSuccessfulUpload,
            self.__CleanFields,
        ]

        self.TorrentClient = Settings.GetTorrentClient()
        self.IncludedFileList = None
        self.VideoFiles: List = []
        self.AdditionalFiles: List = []
        self.MainMediaInfo = None
        self.ReleaseDescription = ""

    def __SetInProgress(self):
        self.ReleaseInfo.JobRunningState = JobRunningState.InProgress
        self.ReleaseInfo.save()

    def __StopAutomaticJobBeforeExtracting(self):
        if (
            self.ReleaseInfo.IsUserCreatedJob()
            or self.ReleaseInfo.AnnouncementSource.StopAutomaticJob
            != "before_extracting"
        ):
            return

        raise PtpUploaderException("Stopping before extracting.")

    def __StopAutomaticJobIfThereAreMultipleVideosBeforeExtracting(self):
        if (
            self.ReleaseInfo.IsUserCreatedJob()
            or self.ReleaseInfo.AnnouncementSource.StopAutomaticJobIfThereAreMultipleVideos
            != "before_extracting"
        ):
            return

        includedFileList = self.ReleaseInfo.AnnouncementSource.GetIncludedFileList(
            self.ReleaseInfo
        )
        self.ReleaseInfo.AnnouncementSource.CheckFileList(
            self.ReleaseInfo, includedFileList
        )

    def __CreateUploadPath(self):
        if self.ReleaseInfo.IsJobPhaseFinished(
            FinishedJobPhase.Upload_CreateUploadPath
        ):
            self.ReleaseInfo.logger().info(
                "Upload path creation phase has been reached previously, not creating it again."
            )
            return

        customUploadPath = self.ReleaseInfo.AnnouncementSource.GetCustomUploadPath(
            self.ReleaseInfo.logger, self.ReleaseInfo
        )
        if len(customUploadPath) > 0:
            self.ReleaseInfo.ReleaseUploadPath = customUploadPath

        self.ReleaseInfo.AnnouncementSource.CreateUploadDirectory(self.ReleaseInfo)

        self.ReleaseInfo.SetJobPhaseFinished(FinishedJobPhase.Upload_CreateUploadPath)
        self.ReleaseInfo.save()

    def __MakeIncludedFileList(self):
        self.IncludedFileList = self.ReleaseInfo.AnnouncementSource.GetIncludedFileList(
            self.ReleaseInfo
        )

        if len(self.ReleaseInfo.IncludedFiles) > 0:
            self.ReleaseInfo.logger().info(
                "There are %s files in the file list. Customized: '%s'.",
                len(self.IncludedFileList.Files),
                self.ReleaseInfo.IncludedFiles,
            )
        else:
            self.ReleaseInfo.logger().info(
                "There are %s files in the file list.", len(self.IncludedFileList.Files)
            )

        self.IncludedFileList.ApplyCustomizationFromJson(self.ReleaseInfo.IncludedFiles)

    def __ExtractRelease(self):
        if self.ReleaseInfo.IsJobPhaseFinished(FinishedJobPhase.Upload_ExtractRelease):
            self.ReleaseInfo.logger().info(
                "Extract release phase has been reached previously, not extracting release again."
            )
            self.VideoFiles, self.AdditionalFiles = parse_directory(self.ReleaseInfo)
            return

        extract_release(self.ReleaseInfo)
        self.VideoFiles, self.AdditionalFiles = parse_directory(self.ReleaseInfo)
        if not self.VideoFiles:
            raise PtpUploaderException(
                "Upload path '%s' doesn't contain any video files."
                % self.ReleaseInfo.GetReleaseUploadPath()
            )
        self.ReleaseInfo.SetJobPhaseFinished(FinishedJobPhase.Upload_ExtractRelease)
        self.ReleaseInfo.save()

    def __GetMediaInfoContainer(self, mediaInfo):
        container = ""

        if mediaInfo.IsAvi():
            container = "AVI"
        elif mediaInfo.IsMkv():
            container = "MKV"
        elif mediaInfo.IsMp4():
            container = "MP4"
            if not self.ReleaseInfo == ReleaseInfo.SourceChoices.HDTV:
                raise PtpUploaderException("MP4 only allowed for HDTV sources")
        elif mediaInfo.IsVob():
            container = "VOB IFO"
        elif mediaInfo.Container == "bdav":
            container = "m2ts"

        if self.ReleaseInfo.Container:
            if container != self.ReleaseInfo.Container:
                if self.ReleaseInfo.IsForceUpload():
                    self.ReleaseInfo.logger().info(
                        "Container is set to '%s', detected MediaInfo container is '%s' ('%s'). Ignoring mismatch because of force upload.",
                        self.ReleaseInfo.Container,
                        container,
                        mediaInfo.Container,
                    )
                else:
                    raise PtpUploaderException(
                        "Container is set to '%s', detected MediaInfo container is '%s' ('%s')."
                        % (self.ReleaseInfo.Container, container, mediaInfo.Container)
                    )
        elif container:
            self.ReleaseInfo.Container = container
        else:
            raise PtpUploaderException(
                "Unsupported container: '%s'." % mediaInfo.Container
            )

    @staticmethod
    def __CanIgnoreDetectedAndSetCodecDifference(detected, given):
        return (detected == "x264" and given == "H.264") or (
            detected == "H.264" and given == "x264"
        )

    def __GetMediaInfoCodec(self, mediaInfo):
        codec = ""

        if mediaInfo.IsX264():
            codec = "x264"
            if mediaInfo.IsAvi():
                raise PtpUploaderException("X264 in AVI is not allowed.")
        elif mediaInfo.IsH264():
            codec = "H.264"
            if mediaInfo.IsAvi():
                raise PtpUploaderException("H.264 in AVI is not allowed.")
        elif mediaInfo.IsXvid():
            codec = "XviD"
            if mediaInfo.IsMkv():
                raise PtpUploaderException("XviD in MKV is not allowed.")
        elif mediaInfo.IsDivx():
            codec = "DivX"
            if mediaInfo.IsMkv():
                raise PtpUploaderException("DivX in MKV is not allowed.")
        elif mediaInfo.IsVc1():
            codec = "VC-1"
        elif mediaInfo.IsMpeg2Codec():
            codec = "MPEG-2"
        elif self.ReleaseInfo.IsDvdImage() or self.ReleaseInfo.IsBlurayImage():
            # Codec type DVD5 and DVD9 can't be figured out from MediaInfo.
            codec = self.ReleaseInfo.Codec

        if self.ReleaseInfo.Codec:
            if codec != self.ReleaseInfo.Codec:
                if Upload.__CanIgnoreDetectedAndSetCodecDifference(
                    codec, self.ReleaseInfo.Codec
                ):
                    if "Remux" not in self.ReleaseInfo.RemasterTitle:
                        self.ReleaseInfo.logger().info(
                            "Codec is set to '%s', detected MediaInfo codec is '%s' ('%s'). Using the detected codec.",
                            self.ReleaseInfo.Codec,
                            codec,
                            mediaInfo.Codec,
                        )
                        self.ReleaseInfo.Codec = codec
                elif self.ReleaseInfo.IsForceUpload():
                    self.ReleaseInfo.logger().info(
                        "Codec is set to '%s', detected MediaInfo codec is '%s' ('%s'). Ignoring mismatch because of force upload.",
                        self.ReleaseInfo.Codec,
                        codec,
                        mediaInfo.Codec,
                    )
                else:
                    raise PtpUploaderException(
                        "Codec is set to '%s', detected MediaInfo codec is '%s' ('%s')."
                        % (self.ReleaseInfo.Codec, codec, mediaInfo.Codec)
                    )
        elif len(codec) > 0:
            self.ReleaseInfo.Codec = codec
        else:
            raise PtpUploaderException("Unsupported codec: '%s'." % mediaInfo.Codec)

    def __GetMediaInfoResolution(self, mediaInfo):
        resolution = ""

        # Indicate the exact resolution for standard definition releases.
        # It is not needed for DVD images.
        if self.ReleaseInfo.IsStandardDefinition() and (
            not self.ReleaseInfo.IsDvdImage()
        ):
            resolution = f"{mediaInfo.Width}x{mediaInfo.Height}"

        if len(self.ReleaseInfo.Resolution) > 0:
            if resolution != self.ReleaseInfo.Resolution:
                if self.ReleaseInfo.IsForceUpload():
                    self.ReleaseInfo.logger().info(
                        "Resolution is set to '%s', detected MediaInfo resolution is '%s' ('%sx%s'). Ignoring mismatch because of force upload.",
                        self.ReleaseInfo.Resolution,
                        resolution,
                        mediaInfo.Width,
                        mediaInfo.Height,
                    )
                else:
                    raise PtpUploaderException(
                        "Resolution is set to '%s', detected MediaInfo resolution is '%s' ('%sx%s')."
                        % (
                            self.ReleaseInfo.Resolution,
                            resolution,
                            mediaInfo.Width,
                            mediaInfo.Height,
                        )
                    )
        else:
            self.ReleaseInfo.Resolution = resolution

    def __MakeReleaseDescription(self):
        self.ReleaseInfo.AnnouncementSource.ReadNfo(self.ReleaseInfo)

        includeReleaseName = (
            self.ReleaseInfo.AnnouncementSource.IncludeReleaseNameInReleaseDescription()
        )
        outputImageDirectory = (
            self.ReleaseInfo.AnnouncementSource.GetTemporaryFolderForImagesAndTorrent(
                self.ReleaseInfo
            )
        )
        releaseDescriptionFormatter = ReleaseDescriptionFormatter(
            self.ReleaseInfo,
            self.VideoFiles,
            self.AdditionalFiles,
            outputImageDirectory,
            not self.ReleaseInfo.OverrideScreenshots,
        )
        self.ReleaseDescription = releaseDescriptionFormatter.Format(includeReleaseName)
        self.MainMediaInfo = releaseDescriptionFormatter.GetMainMediaInfo()

        # To not waste the uploaded screenshots we commit them to the database because the following function calls can all throw exceptions.
        self.ReleaseInfo.save()

        self.__GetMediaInfoContainer(self.MainMediaInfo)
        self.__GetMediaInfoCodec(self.MainMediaInfo)
        self.__GetMediaInfoResolution(self.MainMediaInfo)

    # Returns with true if failed to detect the language.
    def __DetectSubtitlesAddOne(self, subtitleIds, languageName):
        s_id = ptp_subtitle.get_id(languageName)
        if s_id is None:
            # TODO: show warning on the WebUI
            self.ReleaseInfo.logger().warning(
                "Unknown subtitle language: '%s'.", languageName
            )
            return True

        s_id = str(s_id)
        if id not in subtitleIds:
            subtitleIds.append(s_id)

        return False

    def __DetectSubtitles(self):
        subtitleIds = self.ReleaseInfo.Subtitles
        if subtitleIds:
            self.ReleaseInfo.logger().info(
                "Subtitle list is not empty. Skipping subtitle detection."
            )
            return

        self.ReleaseInfo.logger().info("Detecting subtitles.")

        containsUnknownSubtitle = False

        # With DVD images, check if IFO has subtitle language set, if not
        # it becomes much harder to do any detection magic.
        if self.ReleaseInfo.IsDvdImage():
            ifo_path = Path(str(self.MainMediaInfo.Path)[:-5] + "0.IFO")
            if ifo_path.is_file():
                for language in MediaInfo.MediaInfo(
                    self.ReleaseInfo.logger(), ifo_path, ""
                ).Subtitles:
                    containsUnknownSubtitle |= self.__DetectSubtitlesAddOne(
                        subtitleIds, language
                    )
            if not subtitleIds or containsUnknownSubtitle:
                raise PtpUploaderException(
                    "Unable to automatically detect DVD subtitles, please select them manually"
                )

        # Read from MediaInfo.
        for language in self.MainMediaInfo.Subtitles:
            containsUnknownSubtitle |= self.__DetectSubtitlesAddOne(
                subtitleIds, language
            )

        # Try to read from IDX with the same name as the main video file.
        idxPath, _ = os.path.splitext(self.MainMediaInfo.Path)
        idxPath += ".idx"
        if os.path.isfile(idxPath):
            idxLanguages = GetIdxSubtitleLanguages(idxPath)
            if len(idxLanguages) > 0:
                for language in idxLanguages:
                    containsUnknownSubtitle |= self.__DetectSubtitlesAddOne(
                        subtitleIds, language
                    )
            else:
                containsUnknownSubtitle = True

        # If everything went successfully so far, then check if there are any SRT files in the release.
        if not containsUnknownSubtitle:
            for file in self.AdditionalFiles:
                if str(file).lower().endswith(".srt"):
                    # TODO: show warning on the WebUI
                    containsUnknownSubtitle = True
                    break

        if subtitleIds:
            self.ReleaseInfo.SetSubtitles(subtitleIds)
        elif not containsUnknownSubtitle:
            # ID for no subs
            self.ReleaseInfo.SetSubtitles(["44"])

        if not self.ReleaseInfo.Subtitles:
            raise PtpUploaderException(
                "Unable to automatically detect subtitles, please select them manually"
            )

    def __MakeTorrent(self):
        if self.ReleaseInfo.UploadTorrentFilePath:
            self.logger.info(
                "Upload torrent file path is set, not making torrent again."
            )
            return

        # We save it into a separate folder to make sure it won't end up in the upload somehow. :)
        uploadTorrentName = "PTP " + self.ReleaseInfo.ReleaseName + ".torrent"
        uploadTorrentFilePath = (
            self.ReleaseInfo.AnnouncementSource.GetTemporaryFolderForImagesAndTorrent(
                self.ReleaseInfo
            )
        )
        uploadTorrentFilePath = os.path.join(uploadTorrentFilePath, uploadTorrentName)

        uploadTorrentCreatePath = ""

        # Make torrent with the parent directory's name included if there is more than one file or requested by the source (it is a scene release).
        totalFileCount = len(self.VideoFiles) + len(self.AdditionalFiles)
        if totalFileCount > 1 or (
            self.ReleaseInfo.AnnouncementSource.IsSingleFileTorrentNeedsDirectory(
                self.ReleaseInfo
            )
            and not self.ReleaseInfo.ForceDirectorylessSingleFileTorrent
        ):
            uploadTorrentCreatePath = self.ReleaseInfo.GetReleaseUploadPath()
        else:  # Create the torrent including only the single video file.
            uploadTorrentCreatePath = self.MainMediaInfo.Path

        Mktor.Make(self.logger, uploadTorrentCreatePath, uploadTorrentFilePath)

        # Local variables are used temporarily to make sure that values only get stored in the database if MakeTorrent.Make succeeded.
        self.ReleaseInfo.UploadTorrentFilePath = uploadTorrentFilePath
        self.ReleaseInfo.UploadTorrentCreatePath = uploadTorrentCreatePath
        self.ReleaseInfo.save()

    def __CheckIfExistsOnPtp(self):
        # TODO: this is temporary here. We should support it everywhere.
        # If we are not logged in here that could mean that the download took a lot of time and the user got logged out for some reason.
        Ptp.Login()

        # This could be before the Ptp.Login() line, but this way we can hopefully avoid some logging out errors.
        if self.ReleaseInfo.IsZeroImdbId():
            self.logger.info(
                "IMDb ID is set zero, ignoring the check for existing release."
            )
            return

        movieOnPtpResult = None

        if self.ReleaseInfo.PtpId:
            movieOnPtpResult = Ptp.GetMoviePageOnPtp(
                self.logger, self.ReleaseInfo.PtpId
            )
        else:
            movieOnPtpResult = Ptp.GetMoviePageOnPtpByImdbId(
                self.logger, self.ReleaseInfo.ImdbId
            )
            self.ReleaseInfo.PtpId = movieOnPtpResult.PtpId

        # Check (again) if is it already on PTP.
        existingRelease = movieOnPtpResult.IsReleaseExists(self.ReleaseInfo)
        if (
            existingRelease is not None
            and int(existingRelease["Id"]) > self.ReleaseInfo.DuplicateCheckCanIgnore
        ):
            raise PtpUploaderException(
                JobRunningState.DownloadedAlreadyExists,
                "Got uploaded to PTP while we were working on it. Skipping upload because of format '%s'."
                % existingRelease["FullTitle"],
            )

    def __CheckSynopsis(self):
        if (
            Settings.StopIfSynopsisIsMissing
            and Settings.StopIfSynopsisIsMissing.lower() == "before_uploading"
        ):
            self.ReleaseInfo.AnnouncementSource.CheckSynopsis(
                self.logger, self.ReleaseInfo
            )

    def __CheckCoverArt(self):
        self.ReleaseInfo.AnnouncementSource.CheckCoverArt(self.logger, self.ReleaseInfo)

    def __RehostPoster(self):
        # If this movie has no page yet on PTP then we will need the cover, so we rehost the image to an image hoster.
        if self.ReleaseInfo.PtpId or (not self.ReleaseInfo.CoverArtUrl):
            return

        # Rehost the image only if not already on an image hoster.
        url = self.ReleaseInfo.CoverArtUrl
        if "ptpimg.me" in url or "imgbb.co" in url or "catbox.moe" in url:
            return

        self.logger.info("Rehosting poster from '%s'.", url)
        self.ReleaseInfo.CoverArtUrl = ImageHost.upload(self.logger, imageUrl=url)
        self.logger.info("Rehosted poster to '%s'.", self.ReleaseInfo.CoverArtUrl)

    def __StopBeforeUploading(self):
        if self.ReleaseInfo.StopBeforeUploading:
            raise PtpUploaderException("Stopping before uploading.")
        if (
            self.ReleaseInfo.JobStartMode == JobStartMode.Automatic
            and self.ReleaseInfo.AnnouncementSource.StopAutomaticJob
        ):
            raise PtpUploaderException("Stopping automatic job before uploading.")

    def __StartTorrent(self):
        if len(self.ReleaseInfo.UploadTorrentInfoHash) > 0:
            self.logger.info(
                "Upload torrent info hash is set, not starting torrent again."
            )
            return

        # Add torrent without hash checking.
        self.ReleaseInfo.UploadTorrentInfoHash = (
            self.TorrentClient.AddTorrentSkipHashCheck(
                self.logger,
                self.ReleaseInfo.UploadTorrentFilePath,
                self.ReleaseInfo.GetReleaseUploadPath(),
            )
        )
        self.ReleaseInfo.save()

    def __UploadMovie(self):
        # This is not possible because finished jobs can't be restarted.
        if self.ReleaseInfo.IsJobPhaseFinished(FinishedJobPhase.Upload_UploadMovie):
            self.logger.info(
                "Upload movie phase has been reached previously, not uploading it again."
            )
            return

        Ptp.UploadMovie(
            self.logger,
            self.ReleaseInfo,
            self.ReleaseInfo.UploadTorrentFilePath,
            self.ReleaseDescription,
        )

        # Delete the source torrent file.
        if self.ReleaseInfo.SourceTorrentFilePath and os.path.isfile(
            self.ReleaseInfo.SourceTorrentFilePath
        ):
            os.remove(self.ReleaseInfo.SourceTorrentFilePath)
            self.ReleaseInfo.SourceTorrentFilePath = ""

        # Delete the uploaded torrent file.
        if self.ReleaseInfo.UploadTorrentFilePath and os.path.isfile(
            self.ReleaseInfo.UploadTorrentFilePath
        ):
            os.remove(self.ReleaseInfo.UploadTorrentFilePath)
            self.ReleaseInfo.UploadTorrentFilePath = ""

        jobDuration = TimeDifferenceToText(
            datetime.datetime.utcnow() - self.ReleaseInfo.JobStartTimeUtc, 10, "", "0s"
        )
        self.logger.info(
            "'%s' has been successfully uploaded to PTP. Time taken: %s.",
            self.ReleaseInfo.ReleaseName,
            jobDuration,
        )

        self.ReleaseInfo.SetJobPhaseFinished(FinishedJobPhase.Upload_UploadMovie)
        self.ReleaseInfo.JobRunningState = JobRunningState.Finished
        self.ReleaseInfo.save()

    def __ExecuteCommandOnSuccessfulUpload(self):
        # Execute command on successful upload.
        if not Settings.OnSuccessfulUpload:
            return

        django_engine = engines["django"]
        template = django_engine.from_string(Settings.OnSuccessfulUpload)
        context = {}
        for field in self.ReleaseInfo._meta.fields:
            context[field.name] = getattr(self.ReleaseInfo, field.name)
        context["UploadedTorrentUrl"] = (
            "https://passthepopcorn.me/torrents.php?id=" + self.ReleaseInfo.PtpId
        )
        command = template.render(context=context)

        # We don't care if this fails. Our upload is complete anyway. :)
        try:
            subprocess.call(command, shell=True)
        except Exception as e:
            self.ReleaseInfo.logger().exception(
                "Got exception '%s' while trying to run command '%s' after successful upload.",
                e,
                command,
            )

    def __CleanFields(self):
        # Takes up a decent amount of space and can always be recreated if somehow needed later
        self.ReleaseInfo.BdInfo = {}
        self.ReleaseInfo.save()



import logging
import re

from html import unescape

from PtpUploader import ptp_subtitle
from PtpUploader.Helper import GetSizeFromText, ValidateTorrentFile
from PtpUploader.Job.JobRunningState import JobRunningState
from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.PtpUploaderException import (
    PtpUploaderException,
    PtpUploaderInvalidLoginException,
)
from PtpUploader.Source.SourceBase import SourceBase


logger = logging.getLogger(__name__)


class Karagarga(SourceBase):
    def __init__(self):
        super().__init__()

        self.Name = "kg"
        self.NameInSettings = "Karagarga"

    def LoadSettings(self, _):
        super().LoadSettings(_)

        self.AutoUploadDvdImage = self.settings.get("auto_upload_dvd", False)
        self.Username = self.settings.username
        self.Password = self.settings.password

    def Login(self):
        if len(self.Username) <= 0:
            raise PtpUploaderInvalidLoginException(
                "Couldn't log in to Karagarga. Your username is not specified.."
            )

        if len(self.Password) <= 0:
            raise PtpUploaderInvalidLoginException(
                "Couldn't log in to Karagarga. Your password is not specified.."
            )

        logger.info("Logging in to Karagarga.")

        if "karagarga.in" not in MyGlobals.session.cookies.list_domains():
            postData = {"username": self.Username, "password": self.Password}
            result = MyGlobals.session.post(
                "https://karagarga.in/takelogin.php", data=postData
            )
            result.raise_for_status()
            self.__CheckIfLoggedInFromResponse(result.text)
            MyGlobals.SaveCookies()

    def __CheckIfLoggedInFromResponse(self, response: str):
        if (
            response.find('action="takelogin.php"') != -1
            or response.find("""<h2>Login failed!</h2>""") != -1
        ):
            raise PtpUploaderException(
                "Looks like you are not logged in to Karagarga. Probably due to the bad user name or password in settings."
            )

    def __DownloadNfoParseSourceType(self, releaseInfo, description):
        if releaseInfo.Source:
            logger.info(
                "Source '%s' is already set, not getting from the torrent page."
                % releaseInfo.Source
            )
            return

        # <tr><td class="heading" align="right" valign="top">Source</td><td colspan="2" align="left" valign="top">dvdrip</td></tr>
        matches = re.search(
            """<tr><td class="heading".*?>Source</td><td.*?>(.+?)</td></tr>""",
            description,
        )
        if matches is None:
            raise PtpUploaderException(
                JobRunningState.Ignored_MissingInfo,
                "Source type can't be found. Probably the layout of the site has changed.",
            )

        sourceType = matches.group(1).lower()

        if sourceType == "blu-ray":
            releaseInfo.Source = "Blu-ray"
        elif sourceType == "dvd":
            releaseInfo.Source = "DVD"
        elif sourceType == "web":
            releaseInfo.Source = "WEB"
        elif sourceType == "vhs":
            releaseInfo.Source = "VHS"
        elif sourceType == "tv":
            releaseInfo.Source = "TV"
        else:
            raise PtpUploaderException(
                JobRunningState.Ignored_NotSupported,
                "Unsupported source type '%s'." % sourceType,
            )

    @staticmethod
    def __DownloadNfoParseDvdImage(releaseInfo, ripSpecs: str):
        if not releaseInfo.ResolutionType:
            if ripSpecs.find("DVD FORMAT: NTSC") >= 0:
                releaseInfo.ResolutionType = "NTSC"
            elif ripSpecs.find("DVD FORMAT: PAL") >= 0:
                releaseInfo.ResolutionType = "PAL"
            else:
                raise PtpUploaderException(
                    JobRunningState.Ignored_NotSupported,
                    "Can't figure out DVD resolution type from the rip specifications.",
                )

        if ripSpecs.find("VIDEO: ") < 0:
            raise PtpUploaderException(
                JobRunningState.Ignored_NotSupported,
                "DVD video info can't be found in the rip specifications.",
            )

        if ripSpecs.find("AUDIO: ") < 0:
            raise PtpUploaderException(
                JobRunningState.Ignored_NotSupported,
                "DVD audio info can't be found in the rip specifications.",
            )

        untouched_fail = None
        if "video: untouched" not in ripSpecs.lower():
            untouched_fail = "modified video"
        if "audio: untouched" not in ripSpecs.lower():
            untouched_fail = "modified audio"
        if (
            "menus: " in ripSpecs.lower()
            and "menus: untouched" not in ripSpecs.lower()
            and "menus: n/a" not in ripSpecs.lower()
        ):
            untouched_fail = "modified menus"
        if (
            "dvd extras: " in ripSpecs.lower()
            and "dvd extras: n/a" not in ripSpecs.lower()
            and "dvd extras: none" not in ripSpecs.lower()
        ) and "dvd extras: untouched" not in ripSpecs.lower():
            untouched_fail = "modified dvd extras"
        if untouched_fail:
            raise PtpUploaderException(
                JobRunningState.Ignored_NotSupported,
                "The DVD is not untouched: " + untouched_fail,
            )

        if releaseInfo.Size <= 0:
            raise PtpUploaderException(
                JobRunningState.Ignored_NotSupported,
                "Size not set, can't detect DVD's size.",
            )

        # TODO: this isn't correct for multi disc torrents. It must be detected from the file list.
        if releaseInfo.Size > 4707319808:
            releaseInfo.Codec = "DVD9"
        else:
            releaseInfo.Codec = "DVD5"

        releaseInfo.Container = "VOB IFO"

    def __DownloadNfoParseFormatType(self, releaseInfo, description):
        if releaseInfo.Codec:
            logger.info(
                "Codec '%s' is already set, not getting from the torrent page."
                % releaseInfo.Codec
            )
            return

        # <tr><td class="heading" align="right" valign="top">Rip Specs</td><td colspan="2" align="left" valign="top">[General] Format: AVI
        # ...
        # </td></tr>
        ripSpecs = re.search(
            r"<tr><td.*?>Rip Specs</td><td.*?>(.+?)</td></tr>", description, re.DOTALL
        )
        if ripSpecs is None:
            raise PtpUploaderException(
                JobRunningState.Ignored_MissingInfo,
                "Rip specifications can't be found on the page.",
            )

        ripSpecs = ripSpecs.group(1).upper()

        if ripSpecs.find("DVD FORMAT:") >= 0:
            self.__DownloadNfoParseDvdImage(releaseInfo, ripSpecs)
        elif ripSpecs.find("XVID") >= 0:
            releaseInfo.Codec = "XviD"
        elif ripSpecs.find("DIVX") >= 0:
            releaseInfo.Codec = "DivX"
        elif ripSpecs.find("X264") >= 0 or ripSpecs.find("V_MPEG4/ISO/AVC") >= 0:
            releaseInfo.Codec = "x264"
        else:
            raise PtpUploaderException(
                JobRunningState.Ignored_NotSupported,
                "Can't figure out codec from the rip specifications.",
            )

    def __DownloadNfoParseResolution(self, releaseInfo, description):
        if releaseInfo.ResolutionType:
            logger.info(
                "Resolution type '%s' is already set, not getting from the torrent page."
                % releaseInfo.ResolutionType
            )
            return

        if description.find('"genreimages/hdrip720.png"') != -1:
            releaseInfo.ResolutionType = "720p"
        elif description.find('"genreimages/hdrip1080.png"') != -1:
            releaseInfo.ResolutionType = "1080p"
        elif description.find('"genreimages/bluray.png"') != -1:
            raise PtpUploaderException(
                JobRunningState.Ignored_NotSupported,
                "Untouched Blu-ray aren't supported.",
            )
        else:
            # Other HD is in the genre list. It's not supported.
            # <td style="border:none;"><img src="genreimages/dvdr.png" width="40" height="40" border="0" title="DVDR"></td>
            matches = re.search(
                """<td.*?><img src="genreimages/.+?" .*?title="(.+?)".*?></td>""",
                description,
            )
            if matches is not None:
                notSupportedType = matches.group(1).lower()
                if notSupportedType == "hd":
                    raise PtpUploaderException(
                        JobRunningState.Ignored_NotSupported,
                        "Unsupported source or resolution type '%s'."
                        % notSupportedType,
                    )

            releaseInfo.ResolutionType = "Other"

    def __DownloadNfoParseSubtitles(self, releaseInfo, description):
        # Only detect subtitles if they are not specified.
        if releaseInfo.Subtitles:
            return

        # <td class="heading" align="right" valign="top">Subtitles</td><td colspan="2" align="left" valign="top">included: English<hr>
        match = re.search(
            r"<td.+?>Subtitles</td><td.+?>included: (.+?)<hr>", description
        )
        if match is None:
            return

        subtitlesText = match.group(1).lower()

        # Handle specially for subtitle comments like this: "None yet, started working on it.", "No, sorry."
        if subtitlesText.find("none") != -1 or subtitlesText.find("sorry") != -1:
            return

        if subtitlesText in ["no", "no subtitles", "unknown if subtitles included"]:
            return

        # We don't want to add hardcoded subtitles.
        if subtitlesText.find("hard") != -1:
            return

        # On some torrents the subtitle type is indicated too. If it is IDX then we will detect later in a more precise way.
        if subtitlesText.find("idx") != -1 or subtitlesText.find("vobsub") != -1:
            return

        # Remove comments.
        subtitlesText = subtitlesText.replace("subs added separately", "")
        subtitlesText = subtitlesText.replace("(custom)", "")
        subtitlesText = subtitlesText.replace("custom", "")
        subtitlesText = subtitlesText.replace("(optional/softcoded)", "")
        subtitlesText = subtitlesText.replace("(optional)", "")
        subtitlesText = subtitlesText.replace(".srt", "")
        subtitlesText = subtitlesText.replace("srt", "")

        # Go through the list of languages and try to get their PTP IDs.
        subtitleIds = []
        subtitleTexts = subtitlesText.split(",")
        for language in subtitleTexts:
            language = language.strip()
            id = ptp_subtitle.get_id(language)
            if id is None:
                continue

            # IDs are stored strings. And we only add them only once to the list.
            id = str(id)
            if id not in subtitleIds:
                subtitleIds.append(id)

        if len(subtitleIds) > 0:
            releaseInfo.SetSubtitles(subtitleIds)

    def __ParsePage(self, logger, releaseInfo, html, parseForExternalCreateJob=False):
        # Make sure we only get information from the description and not from the comments.
        descriptionEndIndex = html.find('<p><a name="startcomments"></a></p>')
        if descriptionEndIndex == -1:
            raise PtpUploaderException(
                JobRunningState.Ignored_MissingInfo,
                "Description can't found on torrent page. Probably the layout of the site has changed.",
            )

        description = html[:descriptionEndIndex]

        # We will use the torrent's name as release name.
        if not parseForExternalCreateJob:
            matches = re.search(
                r'href="/down.php/(\d+)/.+?">(.+?)\.torrent</a>', description
            )
            if matches is None:
                raise PtpUploaderException(
                    JobRunningState.Ignored_MissingInfo,
                    "Can't get release name from torrent page.",
                )

            releaseName = unescape(matches.group(2))

            # Remove the extension of the container from the release name. (It is there on single file releases.)
            # Optional flags parameter for sub function was only introduced in Python v2.7 so we use compile.sub instead.
            releaseName = re.compile(r"\.avi$", re.IGNORECASE).sub("", releaseName)
            releaseName = re.compile(r"\.mkv$", re.IGNORECASE).sub("", releaseName)
            releaseName = re.compile(r"\.mp4$", re.IGNORECASE).sub("", releaseName)
            if (
                not releaseInfo.ReleaseName
            ) or releaseInfo.ReleaseName == "none":  # "none" can come from FlexGet from the announcement directory.
                releaseInfo.ReleaseName = releaseName

        # Make sure it is under the movie category.
        # <tr><td class="heading" align="right" valign="top">Type</td><td colspan="2" align="left" valign="top"><a href="browse.php?cat=1">Movie</a></td></tr>
        matches = re.search(
            r"""<tr><td.*?>Type</td><td.*?><a href="browse.php\?cat=1">Movie</a></td></tr>""",
            description,
        )
        if matches is None:
            raise PtpUploaderException(
                JobRunningState.Ignored_NotSupported, "Type is not movie."
            )

        # Get IMDb id.
        if (not releaseInfo.ImdbId) and (not releaseInfo.PtpId):
            matches = re.search(r"imdb\.com/title/tt(\d+)", description)
            if matches is None:
                raise PtpUploaderException(
                    JobRunningState.Ignored_MissingInfo,
                    "IMDb id can't be found on torrent page.",
                )

            releaseInfo.ImdbId = matches.group(1)

        # Get size.
        # <tr><td class="heading" align="right" valign="top">Size</td><td colspan="2" align="left" valign="top">1.37GB (1,476,374,914 bytes)</td></tr>
        matches = re.search(
            r"""<tr><td.*?>Size</td><td.*?>.+ \((.+ bytes)\)</td></tr>""", description
        )
        if matches is None:
            logger.warning("Size not found on torrent page.")
        else:
            size = matches.group(1)
            releaseInfo.Size = GetSizeFromText(size)

        self.__DownloadNfoParseSourceType(releaseInfo, description)
        self.__DownloadNfoParseFormatType(releaseInfo, description)
        self.__DownloadNfoParseResolution(releaseInfo, description)
        self.__DownloadNfoParseSubtitles(releaseInfo, description)

        # Make sure that this is not a wrongly categorized DVDR.
        if (not releaseInfo.IsDvdImage()) and (
            re.search(r"<td>.+?\.vob</td>", description, re.IGNORECASE)
            or re.search(r"<td>.+?\.iso</td>", description, re.IGNORECASE)
        ):
            raise PtpUploaderException(
                JobRunningState.Ignored_NotSupported, "Wrongly categorized DVDR."
            )

    def __DownloadNfo(self, logger, releaseInfo):
        url = (
            "https://karagarga.in/details.php?id=%s&filelist=1"
            % releaseInfo.AnnouncementId
        )
        logger.info("Collecting info from torrent page '%s'." % url)

        result = MyGlobals.session.get(url)
        result.raise_for_status()
        response = result.text
        self.__CheckIfLoggedInFromResponse(response)

        self.__ParsePage(logger, releaseInfo, response)

    def __HandleAutoCreatedJob(self, releaseInfo):
        if releaseInfo.IsDvdImage():
            if not self.AutoUploadDvdImage:
                raise PtpUploaderException(
                    JobRunningState.Ignored, "DVD image is on your ignore list."
                )

        # TODO: add filtering support for Karagarga
        # In case of automatic announcement we have to check the release name if it is valid.
        # We know the release name from the announcement, so we can filter it without downloading anything (yet) from the source.
        # if not ReleaseFilter.IsValidReleaseName( releaseInfo.ReleaseName ):
        # 	logger.info( "Ignoring release '%s' because of its name." % releaseInfo.ReleaseName )
        # 	return None

    def PrepareDownload(self, logger, releaseInfo):
        if releaseInfo.IsUserCreatedJob():
            self.__DownloadNfo(logger, releaseInfo)
        else:
            self.__DownloadNfo(logger, releaseInfo)
            self.__HandleAutoCreatedJob(releaseInfo)

    def ParsePageForExternalCreateJob(self, logger, releaseInfo, html):
        self.__ParsePage(logger, releaseInfo, html, parseForExternalCreateJob=True)

    def DownloadTorrent(self, logger, releaseInfo, path):
        # Any non empty filename can be specified.
        url = (
            "https://karagarga.in/down.php/%s/filename.torrent"
            % releaseInfo.AnnouncementId
        )
        logger.info("Downloading torrent file from '%s' to '%s'." % (url, path))

        result = MyGlobals.session.get(url)
        result.raise_for_status()
        response: bytes = result.content
        self.__CheckIfLoggedInFromResponse(response.decode(errors="ignore"))

        with open(path, "wb") as fh:
            fh.write(response)

        ValidateTorrentFile(path)

    def IncludeReleaseNameInReleaseDescription(self):
        return False

    def GetIdFromUrl(self, url):
        result = re.match(r".*karagarga\.in/details.php\?id=(\d+).*", url)
        if result is None:
            return ""
        return result.group(1)

    def GetUrlFromId(self, id):
        return "https://karagarga.in/details.php?id=" + id

    def GetIdFromAutodlIrssiUrl(self, url):
        # https://karagarga.in/down.php/10287/Zhuangzhuang%20Tian%20-%20Lan%20feng%20zheng%20AKA%20The%20Blue%20Kite.torrent
        matches = re.match(r"https?://karagarga\.in/down\.php/(\d+)/.+?\.torrent", url)
        if matches is None:
            return ""
        return matches.group(1)



import datetime
import logging
import os
import threading

from django.utils import timezone

from PtpUploader import Ptp
from PtpUploader.InformationSource.Imdb import Imdb
from PtpUploader.InformationSource.MoviePoster import MoviePoster
from PtpUploader.Job.FinishedJobPhase import FinishedJobPhase
from PtpUploader.Job.WorkerBase import WorkerBase
from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.PtpImdbInfo import PtpImdbInfo, PtpZeroImdbInfo
from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.ReleaseInfo import ReleaseInfo
from PtpUploader.Settings import Settings


logger = logging.getLogger(__name__)


class CheckAnnouncement(WorkerBase):
    def __init__(self, release_id: int, stop_requested: threading.Event):
        super().__init__(release_id, stop_requested)
        self.Phases = [
            self.__CheckAnnouncementSource,
            self.__PrepareDownload,
            self.__CheckSizeLimit,
            self.__ValidateReleaseInfo,
            self.__CheckSchedule,
            self.__CheckIfExistsOnPtp,
            self.__FillOutDetailsForNewMovieByPtpApi,
            self.__FillOutDetailsForNewMovieByExternalSources,
            self.__CheckSynopsis,
            self.__CheckCoverArt,
            self.__CheckImdbRatingAndVoteCount,
            self.__StopAutomaticJobBeforeDownloadingTorrentFile,
        ]

        # Instead of this if, it would be possible to make a totally generic downloader system through SourceBase.
        if self.ReleaseInfo.AnnouncementSourceName == "file":
            self.Phases.append(self.__DetectSceneReleaseFromFileList)
            self.Phases.append(self.__AddToPendingDownloads)
        else:
            self.Phases.extend(
                [
                    self.__CreateReleaseDirectory,
                    self.__DownloadTorrentFile,
                    self.__StopAutomaticJobIfThereAreMultipleVideosBeforeDownloading,
                    self.__DetectSceneReleaseFromFileList,
                    self.__DownloadTorrent,
                    self.__AddToPendingDownloads,
                ]
            )

        self.TorrentClient = Settings.GetTorrentClient()

    def __CheckSchedule(self):
        # User jobs skip the schedule check
        if self.ReleaseInfo.IsUserCreatedJob():
            return
        if self.ReleaseInfo.ScheduleTime == datetime.datetime.fromtimestamp(
            0, timezone.get_default_timezone()
        ):
            delay = self.ReleaseInfo.AnnouncementSource.AutomaticJobStartDelay
            if not delay:
                return
            self.ReleaseInfo.ScheduleTime = timezone.now() + datetime.timedelta(
                minutes=delay
            )
            self.ReleaseInfo.JobRunningState = ReleaseInfo.JobState.Scheduled
            self.ReleaseInfo.ErrorMessage = ""
            self.ReleaseInfo.save()
            raise PtpUploaderException(ReleaseInfo.JobState.Scheduled, "")

    def __CheckAnnouncementSource(self):
        self.ReleaseInfo.logger().info(
            "Working on announcement from '%s' with id '%s' and name '%s'."
            % (
                self.ReleaseInfo.AnnouncementSourceName,
                self.ReleaseInfo.AnnouncementId,
                self.ReleaseInfo.ReleaseName,
            )
        )

        self.ReleaseInfo.JobStartTimeUtc = datetime.datetime.utcnow()
        self.ReleaseInfo.JobRunningState = ReleaseInfo.JobState.InProgress
        self.ReleaseInfo.ErrorMessage = ""
        self.ReleaseInfo.save()

        if (
            self.ReleaseInfo.AnnouncementSource is None
            and self.ReleaseInfo.AnnouncementId is not None
        ):
            source, source_id = MyGlobals.SourceFactory.GetSourceAndIdByUrl(
                self.ReleaseInfo.AnnouncementId
            )
            if source is not None:
                self.ReleaseInfo.AnnouncementSourceName = source.Name
                self.ReleaseInfo.AnnouncementId = source_id
                self.ReleaseInfo.save()
        if self.ReleaseInfo.AnnouncementSource is None:
            raise PtpUploaderException(
                "Announcement source '%s' is unknown."
                % self.ReleaseInfo.AnnouncementSourceName
            )

        if not self.ReleaseInfo.AnnouncementSource.IsEnabled():
            raise PtpUploaderException(
                "Announcement source '%s' is disabled."
                % self.ReleaseInfo.AnnouncementSourceName
            )

    def __PrepareDownload(self):
        self.ReleaseInfo.AnnouncementSource.PrepareDownload(
            self.ReleaseInfo.logger(), self.ReleaseInfo
        )

    def __CheckSizeLimit(self):
        if (
            (not self.ReleaseInfo.IsUserCreatedJob())
            and Settings.SizeLimitForAutoCreatedJobs > 0.0
            and self.ReleaseInfo.Size > Settings.SizeLimitForAutoCreatedJobs
        ):
            raise PtpUploaderException(
                ReleaseInfo.JobState.Ignored, "Ignored because of its size."
            )

    def __ValidateReleaseInfo(self):
        # Make sure we have IMDb or PTP id.
        if (not self.ReleaseInfo.ImdbId) and (not self.ReleaseInfo.PtpId):
            raise PtpUploaderException(
                ReleaseInfo.JobState.Ignored_MissingInfo,
                "IMDb or PTP id must be specified.",
            )

        # Make sure the source is providing a name.
        self.ReleaseInfo.ReleaseName = self.ReleaseInfo.ReleaseName.strip()
        if len(self.ReleaseInfo.ReleaseName) <= 0:
            raise PtpUploaderException(
                ReleaseInfo.JobState.Ignored_MissingInfo,
                "Name of the release is not specified.",
            )

        # Make sure the source is providing release source information.
        if len(self.ReleaseInfo.Source) <= 0:
            raise PtpUploaderException(
                ReleaseInfo.JobState.Ignored_MissingInfo,
                "Source of the release is not specified.",
            )

        # Make sure the source is providing release codec information.
        if len(self.ReleaseInfo.Codec) <= 0:
            raise PtpUploaderException(
                ReleaseInfo.JobState.Ignored_MissingInfo,
                "Codec of the release is not specified.",
            )

        # Make sure the source is providing release resolution type information.
        if len(self.ReleaseInfo.ResolutionType) <= 0:
            raise PtpUploaderException(
                ReleaseInfo.JobState.Ignored_MissingInfo,
                "Resolution type of the release is not specified.",
            )

        # HD XviDs are not allowed.
        if (not self.ReleaseInfo.IsStandardDefinition()) and (
            self.ReleaseInfo.Codec in ["XviD", "DivX"]
        ):
            raise PtpUploaderException(
                ReleaseInfo.JobState.Ignored_Forbidden,
                f"HD {self.ReleaseInfo.Codec}s are not allowed.",
            )

        # HD XviDs are not allowed.
        if (not self.ReleaseInfo.IsStandardDefinition()) and (
            self.ReleaseInfo.Source in ["DVD", "VHS"]
        ):
            raise PtpUploaderException(
                ReleaseInfo.JobState.Ignored_Forbidden,
                f"HD {self.ReleaseInfo.Source} rips are not allowed.",
            )

        # We only support VOB IFO container for DVD images.
        if self.ReleaseInfo.IsDvdImage() and self.ReleaseInfo.Container != "VOB IFO":
            raise PtpUploaderException(
                ReleaseInfo.JobState.Ignored_NotSupported,
                "Only VOB IFO container is supported for DVD images.",
            )

    def __CheckIfExistsOnPtp(self):
        # TODO: this is temporary here. We should support it everywhere.
        # If we are not logged in here that could mean that nothing interesting has been announcened for a while.
        Ptp.Login()

        # This could be before the Ptp.Login() line, but this way we can hopefully avoid some logging out errors.
        if self.ReleaseInfo.IsZeroImdbId():
            self.ReleaseInfo.logger().info(
                "IMDb ID is set zero, ignoring the check for existing release."
            )
            return

        movieOnPtpResult = None

        if self.ReleaseInfo.PtpId:
            movieOnPtpResult = Ptp.GetMoviePageOnPtp(
                self.ReleaseInfo.logger(), self.ReleaseInfo.PtpId
            )

            # If IMDb ID is not set, then store it, because it is needed for renaming releases coming from Cinemageddon and Cinematik.
            if (not self.ReleaseInfo.ImdbId) and len(movieOnPtpResult.ImdbId) > 0:
                self.ReleaseInfo.ImdbId = movieOnPtpResult.ImdbId
        else:
            # Try to get a PTP ID.
            movieOnPtpResult = Ptp.GetMoviePageOnPtpByImdbId(
                self.ReleaseInfo.logger(), self.ReleaseInfo.ImdbId
            )
            self.ReleaseInfo.PtpId = movieOnPtpResult.PtpId

        existingRelease = movieOnPtpResult.IsReleaseExists(self.ReleaseInfo)
        if (
            existingRelease is not None
            and int(existingRelease["Id"]) > self.ReleaseInfo.DuplicateCheckCanIgnore
        ):
            raise PtpUploaderException(
                ReleaseInfo.JobState.Ignored_AlreadyExists,
                "Already exists on PTP: '%s'." % existingRelease["FullTitle"],
            )

        self.ReleaseInfo.ImdbRating = movieOnPtpResult.ImdbRating
        self.ReleaseInfo.ImdbVoteCount = movieOnPtpResult.ImdbVoteCount

    def __FillOutDetailsForNewMovieByPtpApi(self):
        # If already has a page on PTP then we don't have to do anything here.
        if self.ReleaseInfo.PtpId:
            return

        ptpImdbInfo = None
        if self.ReleaseInfo.IsZeroImdbId():
            ptpImdbInfo = PtpZeroImdbInfo()
        else:
            ptpImdbInfo = PtpImdbInfo(self.ReleaseInfo.ImdbId)

        # Title
        if len(self.ReleaseInfo.Title) > 0:
            self.ReleaseInfo.logger().info(
                "Title '%s' is already set, not getting from PTP's movie info."
                % self.ReleaseInfo.Title
            )
        else:
            self.ReleaseInfo.Title = ptpImdbInfo.GetTitle()
            if len(self.ReleaseInfo.Title) <= 0:
                raise PtpUploaderException(
                    ReleaseInfo.JobState.Ignored_MissingInfo, "Movie title is not set."
                )

        # Year
        if len(self.ReleaseInfo.Year) > 0:
            self.ReleaseInfo.logger().info(
                "Year '%s' is already set, not getting from PTP's movie info."
                % self.ReleaseInfo.Year
            )
        else:
            self.ReleaseInfo.Year = ptpImdbInfo.GetYear()
            if len(self.ReleaseInfo.Year) <= 0:
                raise PtpUploaderException(
                    ReleaseInfo.JobState.Ignored_MissingInfo, "Movie year is not set."
                )

        # Movie description
        if len(self.ReleaseInfo.MovieDescription) > 0:
            self.ReleaseInfo.logger().info(
                "Movie description is already set, not getting from PTP's movie info."
            )
        else:
            self.ReleaseInfo.MovieDescription = ptpImdbInfo.GetMovieDescription()

        # Tags
        if len(self.ReleaseInfo.Tags) > 0:
            self.ReleaseInfo.logger().info(
                "Tags '%s' are already set, not getting from PTP's movie info."
                % self.ReleaseInfo.Tags
            )
        else:
            self.ReleaseInfo.Tags = ptpImdbInfo.GetTags()
            if len(self.ReleaseInfo.Tags) <= 0:
                raise PtpUploaderException(
                    ReleaseInfo.JobState.Ignored_MissingInfo,
                    "At least one tag must be specified for a movie.",
                )

        # Cover art URL
        if self.ReleaseInfo.CoverArtUrl:
            self.ReleaseInfo.logger().info(
                "Cover art URL '%s' is already set, not getting from PTP's movie info."
                % self.ReleaseInfo.CoverArtUrl
            )
        else:
            self.ReleaseInfo.CoverArtUrl = ptpImdbInfo.GetCoverArtUrl()

        # Ignore adult movies (if force upload is not set).
        if "adult" in self.ReleaseInfo.Tags:
            if self.ReleaseInfo.IsForceUpload():
                self.ReleaseInfo.logger().info(
                    "Movie's genre is adult, but continuing due to force upload."
                )
            else:
                raise PtpUploaderException(
                    ReleaseInfo.JobState.Ignored_Forbidden, "Genre is adult."
                )

    # Uses IMDb and The Internet Movie Poster DataBase.
    def __FillOutDetailsForNewMovieByExternalSources(self):
        if self.ReleaseInfo.PtpId or self.ReleaseInfo.IsZeroImdbId():
            return

        imdbInfo = Imdb.GetInfo(self.ReleaseInfo.logger(), self.ReleaseInfo.ImdbId)

        # Ignore series (if force upload is not set).
        if imdbInfo.IsSeries:
            if self.ReleaseInfo.IsForceUpload():
                self.ReleaseInfo.logger().info(
                    "The release is a series, but continuing due to force upload."
                )
            else:
                raise PtpUploaderException(
                    ReleaseInfo.JobState.Ignored_Forbidden, "It is a series."
                )

        # PTP returns with the original title, IMDb's iPhone API returns with the international English title.
        self.ReleaseInfo.InternationalTitle = imdbInfo.Title
        if (
            self.ReleaseInfo.Title != self.ReleaseInfo.InternationalTitle
            and len(self.ReleaseInfo.InternationalTitle) > 0
            and self.ReleaseInfo.Title.find(" AKA ") == -1
        ):
            self.ReleaseInfo.Title += " AKA " + self.ReleaseInfo.InternationalTitle

        if len(self.ReleaseInfo.MovieDescription) <= 0:
            self.ReleaseInfo.MovieDescription = imdbInfo.Plot

        if not self.ReleaseInfo.CoverArtUrl:
            self.ReleaseInfo.CoverArtUrl = imdbInfo.PosterUrl
            if not self.ReleaseInfo.CoverArtUrl:
                self.ReleaseInfo.CoverArtUrl = MoviePoster.Get(
                    self.ReleaseInfo.logger(), self.ReleaseInfo.ImdbId
                )

        self.ReleaseInfo.ImdbRating = imdbInfo.ImdbRating
        self.ReleaseInfo.ImdbVoteCount = imdbInfo.ImdbVoteCount

    def __CheckSynopsis(self):
        if (
            Settings.StopIfSynopsisIsMissing
            and Settings.StopIfSynopsisIsMissing.lower() == "before_downloading"
        ):
            self.ReleaseInfo.AnnouncementSource.CheckSynopsis(logger, self.ReleaseInfo)

    def __CheckCoverArt(self):
        if (
            Settings.StopIfCoverArtIsMissing
            and Settings.StopIfCoverArtIsMissing.lower() == "before_downloading"
        ):
            self.ReleaseInfo.AnnouncementSource.CheckCoverArt(logger, self.ReleaseInfo)

    def __IsAllowedImdbRating(self):
        if Settings.StopIfImdbRatingIsLessThan == 0:
            return True

        if len(self.ReleaseInfo.ImdbRating) <= 0:
            return False

        return float(self.ReleaseInfo.ImdbRating) >= float(
            Settings.StopIfImdbRatingIsLessThan
        )

    def __IsAllowedImdbVoteCount(self):
        if Settings.StopIfImdbVoteCountIsLessThan == 0:
            return True

        if len(self.ReleaseInfo.ImdbVoteCount) <= 0:
            return False

        return int(self.ReleaseInfo.ImdbVoteCount) >= int(
            Settings.StopIfImdbVoteCountIsLessThan
        )

    def __CheckImdbRatingAndVoteCount(self):
        # Only applies to automatically created jobs.
        if self.ReleaseInfo.IsUserCreatedJob():
            return

        if not self.__IsAllowedImdbRating():
            raise PtpUploaderException(
                "Ignored because of IMDb rating: %s." % (self.ReleaseInfo.ImdbRating)
            )

        if not self.__IsAllowedImdbVoteCount():
            raise PtpUploaderException(
                "Ignored because of IMDb vote count: %s."
                % (self.ReleaseInfo.ImdbVoteCount)
            )

    def __StopAutomaticJobBeforeDownloadingTorrentFile(self):
        if (
            self.ReleaseInfo.IsUserCreatedJob()
            or self.ReleaseInfo.AnnouncementSource.StopAutomaticJob
            != "before_downloading"
        ):
            return

        raise PtpUploaderException("Stopping before downloading torrent file.")

    def __CreateReleaseDirectory(self):
        if self.ReleaseInfo.IsJobPhaseFinished(
            FinishedJobPhase.Download_CreateReleaseDirectory
        ):
            logger.info(
                "Release root path creation phase has been reached previously, not creating it again."
            )
            return

        releaseRootPath = self.ReleaseInfo.GetReleaseRootPath()
        self.ReleaseInfo.logger().info(
            "Creating release root directory at '%s'." % releaseRootPath
        )

        if os.path.exists(releaseRootPath):
            raise PtpUploaderException(
                "Release root directory '%s' already exists." % releaseRootPath
            )

        os.makedirs(releaseRootPath)

        self.ReleaseInfo.SetJobPhaseFinished(
            FinishedJobPhase.Download_CreateReleaseDirectory
        )
        self.ReleaseInfo.save()

    def __DownloadTorrentFile(self):
        if self.ReleaseInfo.SourceTorrentFilePath:
            self.ReleaseInfo.logger().info(
                "Source torrent file path is set, not downloading the file again."
            )
            return

        torrentName = (
            self.ReleaseInfo.AnnouncementSource.Name
            + " "
            + self.ReleaseInfo.ReleaseName
            + ".torrent"
        )
        sourceTorrentFilePath = os.path.join(
            self.ReleaseInfo.GetReleaseRootPath(), torrentName
        )
        self.ReleaseInfo.AnnouncementSource.DownloadTorrent(
            self.ReleaseInfo.logger(), self.ReleaseInfo, sourceTorrentFilePath
        )

        # Local variable is used temporarily to make sure that SourceTorrentFilePath is only gets stored in the database if DownloadTorrent succeeded.
        self.ReleaseInfo.SourceTorrentFilePath = sourceTorrentFilePath
        self.ReleaseInfo.save()

    def __StopAutomaticJobIfThereAreMultipleVideosBeforeDownloading(self):
        if (
            self.ReleaseInfo.IsUserCreatedJob()
            or self.ReleaseInfo.AnnouncementSource.StopAutomaticJobIfThereAreMultipleVideos
            != "beforedownloading"
        ):
            return

        includedFileList = self.ReleaseInfo.AnnouncementSource.GetIncludedFileList(
            self.ReleaseInfo
        )
        self.ReleaseInfo.AnnouncementSource.CheckFileList(
            self.ReleaseInfo, includedFileList
        )

    def __DetectSceneReleaseFromFileList(self):
        if self.ReleaseInfo.SceneRelease:
            return

        includedFileList = self.ReleaseInfo.AnnouncementSource.GetIncludedFileList(
            self.ReleaseInfo
        )
        self.ReleaseInfo.AnnouncementSource.DetectSceneReleaseFromFileList(
            self.ReleaseInfo, includedFileList
        )

    def __DownloadTorrent(self):
        if len(self.ReleaseInfo.SourceTorrentInfoHash) > 0:
            self.ReleaseInfo.logger().info(
                "Source torrent info hash is set, not starting torent again."
            )
        else:
            self.TorrentClient.CleanTorrentFile(
                self.ReleaseInfo.logger(), self.ReleaseInfo.SourceTorrentFilePath
            )
            self.ReleaseInfo.SourceTorrentInfoHash = self.TorrentClient.AddTorrent(
                logger,
                self.ReleaseInfo.SourceTorrentFilePath,
                self.ReleaseInfo.GetReleaseDownloadPath(),
            )
            self.ReleaseInfo.JobRunningState = ReleaseInfo.JobState.InDownload
            self.ReleaseInfo.save()

    def __AddToPendingDownloads(self):
        self.ReleaseInfo.JobRunningState = ReleaseInfo.JobState.InDownload
        self.ReleaseInfo.save()



import os
import shutil
import time
import xmlrpc.client as xmlrpc_client

import pyrosimple

from pyrosimple.util.metafile import Metafile
from pyrosimple.util.rpc import HashNotFound

from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.PtpUploaderException import PtpUploaderException


class Rtorrent:
    def __init__(self, address):
        MyGlobals.Logger.info("Initializing PyroScope.")
        if address:
            engine = pyrosimple.connect(address)
        else:
            engine = pyrosimple.connect()
        self.proxy = engine.open()

    # downloadPath is the final path. Suggested directory name from torrent won't be added to it.
    # Returns with the info hash of the torrent.
    def AddTorrent(self, logger, torrentPath, downloadPath):
        logger.info(
            "Initiating the download of torrent '%s' with rTorrent to '%s'."
            % (torrentPath, downloadPath)
        )

        metafile = Metafile.from_file(torrentPath)
        metafile.check_meta()

        contents = xmlrpc_client.Binary(metafile.bencode())
        infoHash = metafile.info_hash()

        self.proxy.load.raw_start(
            "",
            contents,
            f'd.directory_base.set="{downloadPath}"',
            "d.custom.set=ptpuploader,true",
        )

        # If load_raw is slow then set_directory_base throws an exception (Fault: <Fault -501: 'Could not find info-hash.'>),
        # so we retry adding the torrent some delay.
        maximumTries = 15
        while True:
            try:
                self.proxy.d.hash(infoHash)
                break
            except HashNotFound:
                if maximumTries > 1:
                    maximumTries -= 1
                    time.sleep(2)  # Two seconds.
                else:
                    raise

        return infoHash

    # Fast resume file is created beside the source torrent with "fast resume " prefix.
    # downloadPath must already contain the data.
    # downloadPath is the final path. Suggested directory name from torrent won't be added to it.
    # Returns with the info hash of the torrent.
    def AddTorrentSkipHashCheck(self, logger, torrentPath, downloadPath):
        logger.info(
            "Adding torrent '%s' without hash checking to rTorrent to '%s'."
            % (torrentPath, downloadPath)
        )

        sourceDirectory, sourceFilename = os.path.split(torrentPath)
        sourceFilename = "fast resume " + sourceFilename
        destinationTorrentPath = os.path.join(sourceDirectory, sourceFilename)

        if os.path.exists(destinationTorrentPath):
            raise PtpUploaderException(
                "Can't create fast resume torrent because path '%s' already exists."
                % destinationTorrentPath
            )

        shutil.copyfile(torrentPath, destinationTorrentPath)

        metafile = Metafile.from_file(destinationTorrentPath)
        metafile.add_fast_resume(downloadPath)
        metafile.save(destinationTorrentPath)

        infoHash = ""
        try:
            infoHash = self.AddTorrent(logger, destinationTorrentPath, downloadPath)
        finally:
            # We always remove the fast resume torrent regardless of result of adding the torrent to rTorrent.
            # This ensures that even if adding to rTorent fails, then resuming the job will work.
            os.remove(destinationTorrentPath)

        return infoHash

    def IsTorrentFinished(self, logger, infoHash):
        # TODO: this try catch block is here because xmlrpclib throws an exception when it timeouts or when the torrent with the given info hash doesn't exists.
        # The latter error most likely will cause stuck downloads so we should add some logic here to cancel an upload. For example: if it haven't download a single byte in ten minutes we can cancel it.

        try:
            # TODO: not the most sophisticated way.
            # Even a watch dir with Pyinotify would be better probably. rTorrent could write the info hash to a directory watched by us.
            return self.proxy.d.complete(infoHash) == 1
        except HashNotFound as e:
            raise e  # Raise this up to the web UI
        except Exception as e:
            logger.exception(
                "Got exception while trying to check torrent's completion status. hash: '%s', error: '%s'",
                infoHash,
                e,
            )

        return False

    # It doesn't delete the data.
    def DeleteTorrent(self, logger, infoHash):
        try:
            self.proxy.d.stop(infoHash)
            self.proxy.d.erase(infoHash)
        except Exception:
            logger.exception(
                "Got exception while trying to delete torrent. Info hash: '%s'."
                % infoHash
            )

    # rTorrent can't download torrents with fast resume information in them, so we clean the torrents before starting the download.
    # This can happen if the uploader uploaded the wrong torrent to the tracker.
    def CleanTorrentFile(self, logger, torrentPath):
        logger.info("Cleaning torrent file '%s'." % torrentPath)

        metafile = Metafile.from_file(torrentPath)
        metafile.clean_meta()
        metafile.save(torrentPath)



# Generated by Django 3.2.5 on 2021-10-31 19:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('web', '0010_alter_releaseinfo_screenshots'),
    ]

    operations = [
        migrations.AlterField(
            model_name='releaseinfo',
            name='Screenshots',
            field=models.JSONField(blank=True, default={}),
        ),
    ]



import os
import re

from pathlib import Path

from PtpUploader.Helper import GetFileListFromTorrent


IMDB_TITLE_RE = re.compile(r"imdb\.com/[Tt]itle/tt(\d+)")


def get_imdb_id(nfo_text: str) -> str:
    match = IMDB_TITLE_RE.search(nfo_text)
    if match:
        return match.group(1)
    return ""


def read_nfo(path: os.PathLike) -> str:
    with Path(path).open("rb") as nfoFile:
        return nfoFile.read().decode("cp437", "ignore")


def find_and_read_nfo(directory: os.PathLike) -> str:
    # If there are multiple NFOs, it returns an empty string
    nfos = list(Path(directory).glob("*.nfo"))
    if len(nfos) != 1:
        return ""
    return read_nfo(nfos[0])


def torrent_has_multiple_nfos(torrent_path: os.PathLike) -> bool:
    found_nfo = False
    for f in GetFileListFromTorrent(torrent_path):
        fpath = Path(f)
        if fpath.parent == "." and fpath.suffix == ".nfo":
            if found_nfo:
                return True
            else:
                found_nfo = True
    return False



import requests

from PtpUploader.ImageHost.Base import ImageSite


class CatboxMoe(ImageSite):
    def __init__(self, logger=None):
        self.name = "catbox"
        self.endpoint = "https://catbox.moe/user/api.php"
        super().__init__(logger)

    def upload_url(self, url: str):
        return self.upload({"userhash": "", "reqtype": "urlupload", "url": url}, {})

    def upload_path(self, path: str):
        with open(path, "rb") as imageHandle:
            return self.upload(
                {"userhash": "", "reqtype": "fileupload"}, {"fileToUpload": imageHandle}
            )

    def upload(self, data, files):
        response = requests.post(self.endpoint, data=data, files=files)
        response.raise_for_status()
        return response.text



import os
import re
import subprocess

from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.Settings import Settings


class Mplayer:
    NoAutoSubParameterSupported = None

    def __init__(self, logger, inputVideoPath):
        Mplayer.DetectNoAutoSubParameterSupport()

        self.Logger = logger
        self.InputVideoPath = inputVideoPath
        self.ScaleSize = None

        self.__CalculateSizeAccordingToAspectRatio()

    # The noautosub parameter is not present in all version of MPlayer, and it returns with error code if it is specified.
    @staticmethod
    def DetectNoAutoSubParameterSupport():
        if Mplayer.NoAutoSubParameterSupported is not None:
            return

        args = [Settings.MplayerPath, "-noautosub"]
        Mplayer.NoAutoSubParameterSupported = (
            subprocess.run(args, check=False).returncode == 0
        )

    # We could get this info when making the first screenshot, but ScaleSize is not stored in the database and screenshots are not made again when resuming a job.
    def __CalculateSizeAccordingToAspectRatio(self):
        args = [
            Settings.MplayerPath,
            "-identify",
            "-vo",
            "null",
            "-frames",
            "1",
            "-nosound",
            "-nosub",
            "-nolirc",
        ]
        if Mplayer.NoAutoSubParameterSupported:
            args.append("-noautosub")
        args.append(self.InputVideoPath)

        proc = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        stdout, stderr = proc.communicate()
        errorCode = proc.wait()
        if errorCode != 0:
            raise PtpUploaderException(
                "Process execution '%s' returned with error code '%s'."
                % (args, errorCode)
            )

        result = stdout.decode("utf-8", "ignore")

        # VO: [null] 1280x720 => 1280x720 RGB 24-bit
        match = re.search(r"VO: \[null\] (\d+)x(\d+) => (\d+)x(\d+).+", result)
        if match is None:
            raise PtpUploaderException("Can't read video size from MPlayer's output.")

        width = int(match.group(1))
        height = int(match.group(2))
        outputWidth = int(match.group(3))
        outputHeight = int(match.group(4))
        if width <= 0 or height <= 0 or outputWidth <= 0 or outputHeight <= 0:
            raise PtpUploaderException("Can't read video size from MPlayer's output.")

        if outputWidth != width or outputHeight != height:
            self.ScaleSize = f"{outputWidth}x{outputHeight}"

    def MakeScreenshotInPng(self, timeInSeconds, outputPngPath):
        self.Logger.info(
            "Making screenshot with MPlayer from '%s' to '%s'."
            % (self.InputVideoPath, outputPngPath)
        )

        # mplayer -ss 101 -vo png:z=9:outdir="/home/tnsuser/temp/a b/" -frames 1 -vf scale=0:0 -nosound -nosub -noautosub -nolirc a.vob
        # outdir is not working with the Windows version of MPlayer, so for the sake of easier testing we set the output directory with by setting the current working directory.
        # -vf scale=0:0 -- use display aspect ratio
        time = str(int(timeInSeconds))
        args = [
            Settings.MplayerPath,
            "-ss",
            time,
            "-vo",
            "png:z=9",
            "-frames",
            "1",
            "-vf",
            "scale=0:0",
            "-nosound",
            "-nosub",
            "-nolirc",
        ]
        if Mplayer.NoAutoSubParameterSupported:
            args.append("-noautosub")
        args.append(self.InputVideoPath)

        errorCode = subprocess.call(args, cwd=Path(outputPngPath).parent)
        if errorCode != 0:
            raise PtpUploaderException(
                "Process execution '%s' returned with error code '%s'."
                % (args, errorCode)
            )

        return outputPngPath



# Generated by Django 3.2.13 on 2022-06-03 16:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('web', '0015_alter_releaseinfo_scheduletime'),
    ]

    operations = [
        migrations.AddField(
            model_name='releaseinfo',
            name='BdInfo',
            field=models.JSONField(blank=True, default=dict),
        ),
    ]
